@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012-2017 Laboratoire de Recherche et Développement
+// Copyright (C) 2012-2018 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -88,11 +88,11 @@ static const argp_option options[] =
     /**************************************************/
     { nullptr, 0, nullptr, 0, "Output format:", 3 },
     { "dot", 'd',
-      "1|a|A|b|B|c|C(COLOR)|e|f(FONT)|h|k|n|N|o|r|R|s|t|v|y|+INT|<INT|#",
+      "1|a|A|b|B|c|C(COLOR)|e|f(FONT)|h|k|K|n|N|o|r|R|s|t|u|v|y|+INT|<INT|#",
       OPTION_ARG_OPTIONAL,
       "GraphViz's format.  Add letters for "
       "(1) force numbered states, "
-      "(a) show acceptance condition, "
+      "(a) show acceptance condition (default), "
       "(A) hide acceptance condition, "
       "(b) acceptance sets as bullets, "
       "(B) bullets except for Büchi/co-Büchi automata, "
@@ -103,6 +103,7 @@ static const argp_option options[] =
       "(f(FONT)) use FONT, "
       "(h) horizontal layout, "
       "(k) use state labels when possible, "
+      "(K) use transition labels (default), "
       "(n) show name, "
       "(N) hide name, "
       "(o) ordered transitions, "
@@ -110,6 +111,7 @@ static const argp_option options[] =
       "(R) color acceptance sets by Inf/Fin, "
       "(s) with SCCs, "
       "(t) force transition-based acceptance, "
+      "(u) hide true states, "
       "(v) vertical layout, "
       "(y) split universal edges by color, "
       "(+INT) add INT to all set numbers, "
