@@ -21,6 +21,7 @@
 #include "common_aoutput.hh"
 
 #include "argp.h"
+#include "closeout.h"
 #include <cstdlib>
 #include <unistd.h>
 #include <iostream>
@@ -32,7 +33,7 @@ static void
 display_version(FILE *stream, struct argp_state*)
 {
   fputs(program_name, stream);
-  fputs(" (" PACKAGE_STRING ")\n\
+  fputs(" (" PACKAGE_NAME ") " PACKAGE_VERSION "\n\
 \n\
 Copyright (C) 2018  Laboratoire de Recherche et Développement de l'Epita.\n\
 License GPLv3+: \
@@ -122,7 +123,6 @@ setup(char** argv)
 // a short version).
 enum {
   OPT_HELP = 1,
-  OPT_USAGE,
   OPT_VERSION,
 };
 
@@ -130,8 +130,9 @@ static const argp_option options[] =
   {
     { "version", OPT_VERSION, nullptr, 0, "print program version", -1 },
     { "help", OPT_HELP, nullptr, 0, "print this help", -1 },
-    // We support this option just in case, but we don't advertise it.
-    { "usage", OPT_USAGE, nullptr, OPTION_HIDDEN, "show short usage", -1 },
+    // We support --usage as a synonym for --help because argp's
+    // hardcoded error message for unknown options mentions it.
+    { "usage", OPT_HELP, nullptr, OPTION_HIDDEN, nullptr, -1 },
     { nullptr, 0, nullptr, 0, nullptr, 0 }
   };
 
@@ -140,8 +141,9 @@ static const argp_option options_hidden[] =
     { "version", OPT_VERSION, nullptr, OPTION_HIDDEN,
       "print program version", -1 },
     { "help", OPT_HELP, nullptr, OPTION_HIDDEN, "print this help", -1 },
-    // We support this option just in case, but we don't advertise it.
-    { "usage", OPT_USAGE, nullptr, OPTION_HIDDEN, "show short usage", -1 },
+    // We support --usage as a synonym for --help because argp's
+    // hardcoded error message for unknown options mentions it.
+    { "usage", OPT_HELP, nullptr, OPTION_HIDDEN, nullptr, -1 },
     { nullptr, 0, nullptr, 0, nullptr, 0 }
   };
 
@@ -152,14 +154,17 @@ parse_opt_misc(int key, char*, struct argp_state* state)
   switch (key)
     {
     case OPT_HELP:
-      argp_state_help(state, state->out_stream, ARGP_HELP_STD_HELP);
-      break;
-    case OPT_USAGE:
       argp_state_help(state, state->out_stream,
-                      ARGP_HELP_USAGE | ARGP_HELP_EXIT_OK);
+                      // Do not let argp exit: we want to diagnose a
+                      // failure to print --help by closing stdout
+                      // properly.
+                      ARGP_HELP_STD_HELP & ~ARGP_HELP_EXIT_OK);
+      close_stdout();
+      exit(0);
       break;
     case OPT_VERSION:
       display_version(state->out_stream, state);
+      close_stdout();
       exit(0);
       break;
     default:
@@ -175,3 +180,23 @@ const struct argp misc_argp = { options, parse_opt_misc,
 const struct argp misc_argp_hidden = { options_hidden, parse_opt_misc,
                                        nullptr, nullptr, nullptr,
                                        nullptr, nullptr };
+
+
+int protected_main(char** progname, std::function<int()> mainfun)
+{
+  try
+    {
+      setup(progname);
+      return mainfun();
+    }
+  catch (const std::runtime_error& e)
+    {
+      error(2, 0, "%s", e.what());
+    }
+  catch (const std::invalid_argument& e)
+    {
+      error(2, 0, "%s", e.what());
+    }
+  SPOT_UNREACHABLE();
+  return 2;
+}
