@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2015, 2016, 2017 Laboratoire de Recherche et
-// Développement de l'Epita (LRDE).
+// Copyright (C) 2015-2018 Laboratoire de Recherche et Développement
+// de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
 //
@@ -44,10 +44,13 @@ struct shorthands_t
   const char* suffix;
 };
 static shorthands_t shorthands_ltl[] = {
+    { "delag", " %f>%O" },
     { "lbt", " <%L>%O" },
     { "ltl2ba", " -f %s>%O" },
     { "ltl2da", " %f>%O" },
+    { "ltl2dgra", " %f>%O" },
     { "ltl2dpa", " %f>%O" },
+    { "ltl2dra", " %f>%O" },
     { "ltl2ldba", " %f>%O" },
     { "ltl2dstar", " --output-format=hoa %[MW]L %O"},
     { "ltl2tgba", " -H %f>%O" },
@@ -64,6 +67,7 @@ static shorthands_t shorthands_autproc[] = {
     { "autfilt", " %H>%O" },
     { "dstar2tgba", " %H>%O" },
     { "ltl2dstar", " -B %H %O" },
+    { "nba2dpa", " <%H>%O" },
     { "nba2ldpa", " <%H>%O" },
     { "seminator", " %H>%O" },
   };
@@ -194,12 +198,6 @@ void tools_push_autproc(const char* proc, bool is_ref)
                      is_ref);
 }
 
-void
-quoted_string::print(std::ostream& os, const char*) const
-{
-  spot::quote_shell_string(os, val().c_str());
-}
-
 void quoted_formula::print(std::ostream& os, const char* pos) const
 {
   spot::formula f = val_;
@@ -312,6 +310,7 @@ filed_automaton::print(std::ostream& os, const char* pos) const
 {
   std::ostringstream ss;
   char* opt = nullptr;
+  bool filename = true;
   if (*pos == '[')
     {
       ++pos;
@@ -330,10 +329,20 @@ filed_automaton::print(std::ostream& os, const char* pos) const
     case 'L':
       spot::print_lbtt(ss, aut_, opt);
       break;
+    case 'M':
+      {
+        filename = false;
+        std::string* name = aut_->get_named_prop<std::string>("automaton-name");
+        spot::quote_shell_string(os,
+                                 name ? name->c_str() :
+                                 opt ? opt :
+                                 "");
+      }
     }
   if (opt)
     free(opt);
-  os << '\'' << string_to_tmp(ss.str(), serial_) << '\'';
+  if (filename)
+    os << '\'' << string_to_tmp(ss.str(), serial_) << '\'';
 }
 
 translator_runner::translator_runner(spot::bdd_dict_ptr dict,
@@ -409,6 +418,7 @@ autproc_runner::autproc_runner(bool no_output_allowed)
   declare('H', &filename_automaton);
   declare('S', &filename_automaton);
   declare('L', &filename_automaton);
+  declare('M', &filename_automaton);
   declare('O', &output);
 
   size_t s = tools.size();
@@ -420,9 +430,10 @@ autproc_runner::autproc_runner(bool no_output_allowed)
       std::vector<bool> has(256);
       const tool_spec& t = tools[n];
       scan(t.cmd, has);
-      if (!(has['H'] || has['S'] || has['L']))
+      if (!(has['H'] || has['S'] || has['L'] || has['M']))
         error(2, 0, "no input %%-sequence in '%s'.\n       Use "
-              "one of %%H,%%S,%%L to indicate the input automaton filename.",
+              "one of %%H,%%S,%%L to indicate the input automaton filename.\n"
+              "        (Or use %%M if you want to work from its name.)\n",
               t.spec);
       if (!no_output_allowed && !has['O'])
         error(2, 0, "no output %%-sequence in '%s'.\n      Use  "
@@ -785,7 +796,7 @@ static int parse_opt_trans(int key, char* arg, struct argp_state*)
       tools_push_trans(arg);
       break;
     case 'T':
-      timeout = to_pos_int(arg);
+      timeout = to_pos_int(arg, "-T/--timeout");
 #if !ENABLE_TIMEOUT
       std::cerr << "warning: setting a timeout is not supported "
                 << "on your platform" << std::endl;
@@ -831,6 +842,8 @@ static const argp_option options_aut[] =
     { "%H,%S,%L", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
       "filename for the input automaton in (H) HOA, (S) Spin's neverclaim, "
       "or (L) LBTT's format", 0 },
+    { "%M, %[val]M", 0, nullptr,  OPTION_DOC | OPTION_NO_USAGE,
+      "the name of the input automaton, with an optional default value", 0 },
     { "%O", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
       "filename for the automaton output in HOA, never claim, LBTT, or "
       "ltl2dstar's format", 0 },
@@ -846,7 +859,7 @@ static int parse_opt_autproc(int key, char* arg, struct argp_state*)
       tools_push_autproc(arg);
       break;
     case 'T':
-      timeout = to_pos_int(arg);
+      timeout = to_pos_int(arg, "-T/--timeout");
 #if !ENABLE_TIMEOUT
       std::cerr << "warning: setting a timeout is not supported "
                 << "on your platform" << std::endl;
