@@ -40,10 +40,10 @@
 #include <spot/misc/timer.hh>
 
 static const char argp_program_doc[] ="\
-Translate linear-time formulas (LTL/PSL) into Büchi automata.\n\n\
+Translate linear-time formulas (LTL/PSL) into various types of automata.\n\n\
 By default it will apply all available optimizations to output \
 the smallest Transition-based Generalized Büchi Automata, \
-in GraphViz's format.\n\
+output in the HOA format.\n\
 If multiple formulas are supplied, several automata will be output.";
 
 
@@ -158,23 +158,20 @@ namespace
 int
 main(int argc, char** argv)
 {
-  // By default we name automata using the formula.
-  opt_name = "%f";
+  return protected_main(argv, [&] {
+      // By default we name automata using the formula.
+      opt_name = "%f";
 
-  setup(argv);
+      const argp ap = { options, parse_opt, "[FORMULA...]",
+                        argp_program_doc, children, nullptr, nullptr };
 
-  const argp ap = { options, parse_opt, "[FORMULA...]",
-                    argp_program_doc, children, nullptr, nullptr };
+      simplification_level = 3;
 
-  simplification_level = 3;
+      if (int err = argp_parse(&ap, argc, argv, ARGP_NO_HELP, nullptr, nullptr))
+        exit(err);
 
-  if (int err = argp_parse(&ap, argc, argv, ARGP_NO_HELP, nullptr, nullptr))
-    exit(err);
+      check_no_formula();
 
-  check_no_formula();
-
-  try
-    {
       spot::translator trans(&extra_options);
       trans.set_type(type);
       trans.set_pref(pref | comp | sbacc | unambig | colored);
@@ -186,11 +183,6 @@ main(int argc, char** argv)
 
       // Diagnose unused -x options
       extra_options.report_unused_options();
-    }
-  catch (const std::runtime_error& e)
-    {
-      error(2, 0, "%s", e.what());
-    }
-
-  return 0;
+      return 0;
+    });
 }
