@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017 Laboratoire de Recherche et
-// Développement de l'Epita (LRDE).
+// Copyright (C) 2012-2018 Laboratoire de Recherche et Développement
+// de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
 //
@@ -83,6 +83,7 @@ enum {
   OPT_IGNORE_ERRORS,
   OPT_IMPLIED_BY,
   OPT_IMPLY,
+  OPT_LIVENESS,
   OPT_LTL,
   OPT_NEGATE,
   OPT_NNF,
@@ -100,6 +101,7 @@ enum {
   OPT_SIZE_MIN,
   OPT_SKIP_ERRORS,
   OPT_STUTTER_INSENSITIVE,
+  OPT_SUSPENDABLE,
   OPT_SYNTACTIC_GUARANTEE,
   OPT_SYNTACTIC_OBLIGATION,
   OPT_SYNTACTIC_PERSISTENCE,
@@ -167,6 +169,8 @@ static const argp_option options[] =
     { "eventual", OPT_EVENTUAL, nullptr, 0, "match pure eventualities", 0 },
     { "universal", OPT_UNIVERSAL, nullptr, 0,
       "match purely universal formulas", 0 },
+    { "suspendable", OPT_SUSPENDABLE, nullptr, 0,
+      "synonym for --universal --eventual", 0 },
     { "syntactic-safety", OPT_SYNTACTIC_SAFETY, nullptr, 0,
       "match syntactic-safety formulas", 0 },
     { "syntactic-guarantee", OPT_SYNTACTIC_GUARANTEE, nullptr, 0,
@@ -180,6 +184,8 @@ static const argp_option options[] =
     { "syntactic-stutter-invariant", OPT_SYNTACTIC_SI, nullptr, 0,
       "match stutter-invariant formulas syntactically (LTL-X or siPSL)", 0 },
     { "nox", 0, nullptr, OPTION_ALIAS, nullptr, 0 },
+    { "liveness", OPT_LIVENESS, nullptr, 0,
+      "match liveness properties", 0 },
     { "safety", OPT_SAFETY, nullptr, 0,
       "match safety formulas (even pathological)", 0 },
     { "guarantee", OPT_GUARANTEE, nullptr, 0,
@@ -278,6 +284,7 @@ static bool negate = false;
 static bool boolean_to_isop = false;
 static bool unique = false;
 static bool psl = false;
+static bool liveness = false;
 static bool ltl = false;
 static bool invert = false;
 static bool boolean = false;
@@ -345,7 +352,7 @@ parse_opt(int key, char* arg, struct argp_state*)
       output_format = count_output;
       break;
     case 'n':
-      opt_max_count = to_pos_int(arg);
+      opt_max_count = to_pos_int(arg, "-n/--max-count");
       break;
     case 'q':
       output_format = quiet_output;
@@ -385,10 +392,10 @@ parse_opt(int key, char* arg, struct argp_state*)
       bsize = parse_range(arg, 0, std::numeric_limits<int>::max());
       break;
     case OPT_BSIZE_MIN:
-      bsize.min = to_int(arg);
+      bsize.min = to_int(arg, "--bsize-min");
       break;
     case OPT_BSIZE_MAX:
-      bsize.max = to_int(arg);
+      bsize.max = to_int(arg, "--bsize-max");
       break;
     case OPT_DEFINE:
       opt->output_define.reset(new output_file(arg ? arg : "-"));
@@ -429,6 +436,9 @@ parse_opt(int key, char* arg, struct argp_state*)
         opt->imply = spot::formula::And({opt->imply, i});
         break;
       }
+    case OPT_LIVENESS:
+      liveness = true;
+      break;
     case OPT_LTL:
       ltl = true;
       break;
@@ -487,10 +497,10 @@ parse_opt(int key, char* arg, struct argp_state*)
       size = parse_range(arg, 0, std::numeric_limits<int>::max());
       break;
     case OPT_SIZE_MIN:
-      size.min = to_int(arg);
+      size.min = to_int(arg, "--size-min");
       break;
     case OPT_SIZE_MAX:
-      size.max = to_int(arg);
+      size.max = to_int(arg, "--size-max");
       break;
     case OPT_SKIP_ERRORS:
       error_style = skip_errors;
@@ -507,6 +517,10 @@ parse_opt(int key, char* arg, struct argp_state*)
     case OPT_AP_N:
       ap_n = parse_range(arg, 0, std::numeric_limits<int>::max());
       break;
+    case OPT_SUSPENDABLE:
+      universal = true;
+      eventual = true;
+      break;
     case OPT_SYNTACTIC_SAFETY:
       syntactic_safety = true;
       break;
@@ -690,7 +704,7 @@ namespace
         {
           spot::twa_graph_ptr aut = nullptr;
 
-          if (!opt->acc_words.empty() && !opt->rej_words.empty())
+          if (!opt->acc_words.empty() || !opt->rej_words.empty())
             {
               aut = ltl_to_tgba_fm(f, simpl.get_dict(), true);
 
@@ -781,7 +795,8 @@ namespace
               for (auto& p: m)
                 stream_formula(opt->output_define->ostream()
                                << "#define " << p.first << " (",
-                               p.second, filename, linenum) << ")\n";
+                               p.second, filename,
+                               std::to_string(linenum).c_str()) << ")\n";
             }
           one_match = true;
           output_formula_checked(f, &timer, filename, linenum, prefix, suffix);
@@ -795,13 +810,10 @@ namespace
 int
 main(int argc, char** argv)
 {
-  setup(argv);
+  return protected_main(argv, [&] {
+      const argp ap = { options, parse_opt, "[FILENAME[/COL]...]",
+                        argp_program_doc, children, nullptr, nullptr };
 
-  const argp ap = { options, parse_opt, "[FILENAME[/COL]...]",
-                    argp_program_doc, children, nullptr, nullptr };
-
-  try
-    {
       // This will ensure that all objects stored in this struct are
       // destroyed before global variables.
       opt_t o;
@@ -822,19 +834,10 @@ main(int argc, char** argv)
       ltl_processor processor(simpl);
       if (processor.run())
         return 2;
-    }
-  catch (const std::runtime_error& e)
-    {
-      error(2, 0, "%s", e.what());
-    }
-  catch (const std::invalid_argument& e)
-    {
-      error(2, 0, "%s", e.what());
-    }
 
-  if (output_format == count_output)
-    std::cout << match_count << std::endl;
-  flush_cout();
-
-  return one_match ? 0 : 1;
+      if (output_format == count_output)
+        std::cout << match_count << '\n';
+      flush_cout();
+      return one_match ? 0 : 1;
+    });
 }
