@@ -143,10 +143,13 @@ int fdd_extdomain(int *dom, int num)
       {
          fdvaralloc += (num > fdvaralloc) ? num : fdvaralloc;
 
-	 Domain* tmp_ptr = (Domain*)realloc(domain, sizeof(Domain)*fdvaralloc);
-	 if (tmp_ptr == NULL)
-	    return bdd_error(BDD_MEMORY);
-	 domain = tmp_ptr;
+         {
+           Domain* tmp_ptr =
+             (Domain*)realloc(domain, sizeof(Domain)*fdvaralloc);
+           if (tmp_ptr == NULL)
+             return bdd_error(BDD_MEMORY);
+           domain = tmp_ptr;
+         }
       }
    }
 
@@ -220,10 +223,12 @@ int fdd_overlapdomain(int v1, int v2)
    {
       fdvaralloc += fdvaralloc;
 
-      Domain* tmp_ptr = (Domain*)realloc(domain, sizeof(Domain)*fdvaralloc);
-      if (tmp_ptr == NULL)
-	 return bdd_error(BDD_MEMORY);
-      domain = tmp_ptr;
+      {
+        Domain* tmp_ptr = (Domain*)realloc(domain, sizeof(Domain)*fdvaralloc);
+        if (tmp_ptr == NULL)
+          return bdd_error(BDD_MEMORY);
+        domain = tmp_ptr;
+      }
    }
 
    d = &domain[fdvarnum];
@@ -949,6 +954,8 @@ int fdd_intaddvarblock(int first, int last, int fixed)
       res = tmp;
    }
 
+   bdd_addref(res); /* Added by Jaco van de Pol, 25 march 2010 */
+
    err = bdd_addvarblock(res, fixed);
 
    bdd_delref(res);
