@@ -1,5 +1,5 @@
 /*========================================================================
-	       Copyright (C) 1996-2002, 2015 by Jorn Lind-Nielsen
+	       Copyright (C) 1996-2002 by Jorn Lind-Nielsen
 			    All rights reserved
 
     Permission is hereby granted, without written agreement and without
@@ -101,6 +101,8 @@ int*         bddvar2level;      /* Variable -> level table */
 int*         bddlevel2var;      /* Level -> variable table */
 jmp_buf      bddexception;      /* Long-jump point for interrupting calc. */
 int          bddresized;        /* Flag indicating a resize of the nodetable */
+int          bddcachesize;      /* Size of the operator caches */
+int          bddhashsize;       /* Size of the BDD node hash */
 
 bddCacheStat bddcachestats;
 
@@ -109,7 +111,6 @@ bddCacheStat bddcachestats;
 
 static BDD*     bddvarset;             /* Set of defined BDD variables */
 static int      gbcollectnum;          /* Number of garbage collections */
-static int      cachesize;             /* Size of the operator caches */
 static long int gbcclock;              /* Clock ticks used in GBC */
 static int      usednodes_nextreorder; /* When to do reorder next time */
 static bddinthandler  err_handler;     /* Error handler */
@@ -140,7 +141,7 @@ static const char *errorstrings[BDD_ERRNUM] =
 
 /*=== OTHER INTERNAL DEFINITIONS =======================================*/
 
-#define NODEHASH(lvl,l,h) (TRIPLE(lvl,l,h) % bddnodesize)
+#define NODEHASH(lvl,l,h) (TRIPLE(lvl,l,h) & (bddhashsize - 1))
 
 
 /*************************************************************************
@@ -179,12 +180,13 @@ int bdd_init(int initnodesize, int cs)
    if (bddrunning)
       return bdd_error(BDD_RUNNING);
 
-   bddnodesize = bdd_prime_gte(initnodesize);
+   bddnodesize = initnodesize;
 
    if ((bddnodes=(BddNode*)malloc(sizeof(BddNode)*bddnodesize)) == NULL)
       return bdd_error(BDD_MEMORY);
 
-   if ((bddhash=(int*)calloc(bddnodesize, sizeof(*bddhash))) == NULL)
+   bddhashsize = bdd_nextpower(bddnodesize);
+   if ((bddhash=(int*)calloc(bddhashsize, sizeof(*bddhash))) == NULL)
      {
        free(bddnodes);
        return bdd_error(BDD_MEMORY);
@@ -192,15 +194,23 @@ int bdd_init(int initnodesize, int cs)
 
    bddresized = 0;
 
-   for (n=0 ; n<bddnodesize ; n++)
+   /* Load these globals into local variables to help the
+      optimizer. */
    {
-      bddnodes[n].refcou = 0;
-      LOW(n) = -1;
-      LEVEL(n) = 0;
-      bddnodes[n].next = n+1;
+     int sz = bddnodesize;
+     BddNodeInit* b = (BddNodeInit*)bddnodes;
+     for (n=0 ; n<sz ; n++)
+       {
+         b[n].z = 0;
+         b[n].low = -1;
+         /* Initializing HIGH is useless in BuDDy, but it helps the
+            compiler vectorizing the code, and it helps the processor
+            write-combining data to memory. */
+         b[n].high = 0;
+         b[n].next = n+1;
+       }
+     b[sz-1].next = 0;
    }
-   bddnodes[bddnodesize-1].next = 0;
-
    bddnodes[0].refcou = bddnodes[1].refcou = MAXREF;
    LOW(0) = HIGH(0) = 0;
    LOW(1) = HIGH(1) = 1;
@@ -217,7 +227,7 @@ int bdd_init(int initnodesize, int cs)
    bddvarnum = 0;
    gbcollectnum = 0;
    gbcclock = 0;
-   cachesize = cs;
+   bddcachesize = cs;
    usednodes_nextreorder = bddnodesize;
    bddmaxnodeincrease = DEFAULTMAXNODEINC;
 
@@ -334,25 +344,31 @@ int bdd_setvarnum(int num)
    }
    else
    {
-     BDD* tmp_ptr = (BDD*)realloc(bddvarset,sizeof(BDD)*num*2);
-     if (__unlikely(tmp_ptr == NULL))
-       return bdd_error(BDD_MEMORY);
-     bddvarset = tmp_ptr;
-     int* tmp_ptr2 = (int*)realloc(bddlevel2var,sizeof(int)*(num+1));
-     if (__unlikely(tmp_ptr2 == NULL))
-       {
-	 free(bddvarset);
-	 return bdd_error(BDD_MEMORY);
-       }
-     bddlevel2var = tmp_ptr2;
-     tmp_ptr2 = (int*)realloc(bddvar2level,sizeof(int)*(num+1));
-     if (__unlikely(tmp_ptr2 == NULL))
-      {
-	 free(bddvarset);
-	 free(bddlevel2var);
-	 return bdd_error(BDD_MEMORY);
-      }
-     bddvar2level = tmp_ptr2;
+     {
+       BDD* tmp_ptr = (BDD*)realloc(bddvarset,sizeof(BDD)*num*2);
+       if (__unlikely(tmp_ptr == NULL))
+         return bdd_error(BDD_MEMORY);
+       bddvarset = tmp_ptr;
+     }
+     {
+       int* tmp_ptr2 = (int*)realloc(bddlevel2var,sizeof(int)*(num+1));
+       if (__unlikely(tmp_ptr2 == NULL))
+         {
+           free(bddvarset);
+           return bdd_error(BDD_MEMORY);
+         }
+       bddlevel2var = tmp_ptr2;
+     }
+     {
+       int* tmp_ptr2 = (int*)realloc(bddvar2level,sizeof(int)*(num+1));
+       if (__unlikely(tmp_ptr2 == NULL))
+         {
+           free(bddvarset);
+           free(bddlevel2var);
+           return bdd_error(BDD_MEMORY);
+         }
+       bddvar2level = tmp_ptr2;
+     }
    }
 
    if (__likely(bddrefstack != NULL))
@@ -722,7 +738,8 @@ void bdd_stats(bddStat *s)
    s->freenodes = bddfreenum;
    s->minfreenodes = minfreenodes;
    s->varnum = bddvarnum;
-   s->cachesize = cachesize;
+   s->cachesize = bddcachesize;
+   s->hashsize = bddhashsize;
    s->gbcnum = gbcollectnum;
 }
 
@@ -769,19 +786,19 @@ void bdd_fprintstat(FILE *ofile)
    fprintf(ofile, "\nCache statistics\n");
    fprintf(ofile, "----------------\n");
 
-   fprintf(ofile, "Unique Access:  %ld\n", s.uniqueAccess);
-   fprintf(ofile, "Unique Chain:   %ld\n", s.uniqueChain);
-   fprintf(ofile, "Unique Hit:     %ld\n", s.uniqueHit);
-   fprintf(ofile, "Unique Miss:    %ld\n", s.uniqueMiss);
+   fprintf(ofile, "Unique Access:  %lu\n", s.uniqueAccess);
+   fprintf(ofile, "Unique Chain:   %lu\n", s.uniqueChain);
+   fprintf(ofile, "Unique Hit:     %lu\n", s.uniqueHit);
+   fprintf(ofile, "Unique Miss:    %lu\n", s.uniqueMiss);
    fprintf(ofile, "=> Hit rate =   %.2f\n",
 	   (s.uniqueHit+s.uniqueMiss > 0) ?
 	   ((float)s.uniqueHit)/((float)s.uniqueHit+s.uniqueMiss) : 0);
-   fprintf(ofile, "Operator Hits:  %ld\n", s.opHit);
-   fprintf(ofile, "Operator Miss:  %ld\n", s.opMiss);
+   fprintf(ofile, "Operator Hits:  %lu\n", s.opHit);
+   fprintf(ofile, "Operator Miss:  %lu\n", s.opMiss);
    fprintf(ofile, "=> Hit rate =   %.2f\n",
 	   (s.opHit+s.opMiss > 0) ?
 	   ((float)s.opHit)/((float)s.opHit+s.opMiss) : 0);
-   fprintf(ofile, "Swap count =    %ld\n", s.swapCount);
+   fprintf(ofile, "Swap count =    %lu\n", s.swapCount);
 }
 
 
@@ -1066,7 +1083,7 @@ void bdd_gbc(void)
 	 bdd_mark(n);
    }
 
-   memset(bddhash, 0, bddnodesize*sizeof(*bddhash));
+   memset(bddhash, 0, bddhashsize*sizeof(*bddhash));
 
    bddfreepos = 0;
    bddfreenum = 0;
@@ -1133,7 +1150,7 @@ SHORT   {* increases the reference count on a node *}
 PROTO   {* BDD bdd_addref(BDD r) *}
 DESCR   {* Reference counting is done on externaly referenced nodes only
 	   and the count for a specific node {\tt r} can and must be
-	   increased using this function to avoid loosing the node in the next
+	   increased using this function to avoid losing the node in the next
 	   garbage collection. *}
 ALSO    {* bdd\_delref *}
 RETURN  {* The BDD node {\tt r}. *}
@@ -1155,11 +1172,11 @@ BDD bdd_addref(BDD root)
    return root;
 }
 
-// Non constant version
+/* Non constant version */
 BDD bdd_delref_nc(BDD root)
 {
 #ifndef NDEBUG
-   if (root < 2  ||  !bddrunning)
+   if (!bddrunning)
       return root;
    if (root >= bddnodesize)
       return bdd_error(BDD_ILLBDD);
@@ -1347,7 +1364,7 @@ int bdd_makenode(unsigned int level, int low, int high)
 #endif
 
       /* Any free nodes to use ? */
-   if (bddfreepos == 0)
+   if (__unlikely(bddfreepos == 0))
    {
       if (bdderrorcond)
 	 return 0;
@@ -1399,12 +1416,13 @@ int bdd_noderesize(int doRehash)
 {
    BddNode *newnodes;
    int oldsize = bddnodesize;
+   int oldhashsize = bddhashsize;
    int n;
 
    if (bddnodesize >= bddmaxnodesize  &&  bddmaxnodesize > 0)
       return -1;
 
-   bddnodesize = bddnodesize << 1;
+   bddnodesize <<= 1;
 
    if (bddnodesize > oldsize + bddmaxnodeincrease)
       bddnodesize = oldsize + bddmaxnodeincrease;
@@ -1412,8 +1430,6 @@ int bdd_noderesize(int doRehash)
    if (bddnodesize > bddmaxnodesize  &&  bddmaxnodesize > 0)
       bddnodesize = bddmaxnodesize;
 
-   bddnodesize = bdd_prime_lte(bddnodesize);
-
    if (resize_handler != NULL)
       resize_handler(oldsize, bddnodesize);
 
@@ -1422,33 +1438,40 @@ int bdd_noderesize(int doRehash)
       return bdd_error(BDD_MEMORY);
    bddnodes = newnodes;
 
-   /* An error while reallocating bddhash is very unlikely, because
-      the new bddhash should fit easily in the area freed by the old
-      bddnode.  */
+   if (oldhashsize * 2 <= bddnodesize)
+     bddhashsize <<= 1;
+
    if (doRehash)
      {
        free(bddhash);
-       if ((bddhash=(int*)calloc(bddnodesize, sizeof(*bddhash))) == NULL)
+       if ((bddhash=(int*)calloc(bddhashsize, sizeof(*bddhash))) == NULL)
 	 return bdd_error(BDD_MEMORY);
      }
    else
      {
-       bddhash = (int*)realloc(bddhash, sizeof(*bddhash)*bddnodesize);
+       bddhash = (int*)realloc(bddhash, sizeof(*bddhash)*bddhashsize);
        if (bddhash == NULL)
 	 return bdd_error(BDD_MEMORY);
-       memset(bddhash + oldsize, 0, (bddnodesize-oldsize)*sizeof(*bddhash));
+       memset(bddhash + oldhashsize, 0,
+              (bddhashsize-oldhashsize)*sizeof(*bddhash));
      }
 
-   for (n=oldsize ; n<bddnodesize ; n++)
+   /* copy these global variables into local variables to help the
+      optimizer */
    {
-      bddnodes[n].refcou = 0;
-      LEVEL(n) = 0;
-      LOW(n) = -1;
-      bddnodes[n].next = n+1;
+     int sz = bddnodesize;
+     BddNodeInit* b = (BddNodeInit*)(bddnodes);
+     for (n=oldsize ; n<sz ; n++)
+       {
+         b[n].z = 0;
+         b[n].low = -1;
+         b[n].high = 0;
+         b[n].next = n+1;
+       }
+     b[sz-1].next = bddfreepos;
+     bddfreepos = oldsize;
+     bddfreenum += bddnodesize - oldsize;
    }
-   bddnodes[bddnodesize-1].next = bddfreepos;
-   bddfreepos = oldsize;
-   bddfreenum += bddnodesize - oldsize;
 
    if (doRehash)
       bdd_gbc_rehash();
