@@ -99,6 +99,15 @@ typedef struct s_BddNode /* Node table entry */
    int next;
 } BddNode;
 
+/* This structure is used during initialization of the above
+   structure, to help gcc vectorize the initializing loop. */
+typedef struct s_BddNodeInit
+{
+   int z;
+   int low;
+   int high;
+   int next;
+} BddNodeInit;
 
 /*=== KERNEL VARIABLES =================================================*/
 
@@ -109,6 +118,7 @@ extern "C" {
 extern int       bddrunning;         /* Flag - package initialized */
 extern int       bdderrorcond;       /* Some error condition was met */
 extern int       bddnodesize;        /* Number of allocated nodes */
+extern int       bddhashsize;        /* Size of node hash tableq */
 extern int       bddmaxnodesize;     /* Maximum allowed number of nodes */
 extern int       bddmaxnodeincrease; /* Max. # of nodes used to inc. table */
 extern BddNode*  bddnodes;           /* All of the bdd nodes */
@@ -121,8 +131,12 @@ extern int*      bddlevel2var;
 extern jmp_buf   bddexception;
 extern int       bddreorderdisabled;
 extern int       bddresized;
+extern int       bddcachesize;
 extern bddCacheStat bddcachestats;
 
+  /* from reorder.c */
+extern int bddreordermethod;
+
 #ifdef CPLUSPLUS
 }
 #endif
