@@ -33,6 +33,7 @@ from spot.aux import \
 import subprocess
 import os
 import signal
+import tempfile
 from contextlib import suppress as _supress
 
 
@@ -192,6 +193,9 @@ class formula:
         from IPython.display import SVG
         return SVG(_str_to_svg(self.to_str('d')))
 
+    def _repr_latex_(self):
+        return '$' + self.to_str('j') + '$'
+
     def to_str(self, format='spot', parenth=False):
         if format == 'spot' or format == 'f':
             return str_psl(self, parenth)
@@ -207,6 +211,10 @@ class formula:
             return str_latex_psl(self, parenth)
         elif format == 'sclatex' or format == 'X':
             return str_sclatex_psl(self, parenth)
+        elif format == 'mathjax' or format == 'j':
+            return (str_sclatex_psl(self, parenth).
+                        replace("``", "\\unicode{x201C}").
+                        replace("\\textrm{''}", "\\unicode{x201D}"))
         elif format == 'dot' or format == 'd':
             ostr = ostringstream()
             print_dot_psl(ostr, self)
@@ -233,6 +241,7 @@ class formula:
         - 'w': use Wring's syntax
         - 'x': use LaTeX output
         - 'X': use self-contained LaTeX output
+        - 'j': use self-contained LaTeX output, adjusted for MathJax
 
         Add some of those letters for additional options:
 
@@ -262,7 +271,7 @@ class formula:
 
         while spec:
             c, spec = spec[0], spec[1:]
-            if c in ('f', 's', '8', 'l', 'w', 'x', 'X'):
+            if c in ('f', 's', '8', 'l', 'w', 'x', 'X', 'j'):
                 syntax = c
             elif c == 'p':
                 parent = True
@@ -323,6 +332,35 @@ class formula:
         raise ValueError("unknown type of formula")
 
 
+@_extend(atomic_prop_set)
+class atomic_prop_set:
+    def __repr__(self):
+        res = '{'
+        comma = ''
+        for ap in self:
+            res += comma
+            comma = ', '
+            res += '"' + ap.ap_name() + '"'
+        res += '}'
+        return res
+
+    def __str__(self):
+        return self.__repr__()
+
+    def _repr_latex_(self):
+        res = '$\{'
+        comma = ''
+        for ap in self:
+            apname = ap.to_str('j')
+            if not '\\unicode{' in apname:
+                apname = "\\unicode{x201C}" + apname + "\\unicode{x201D}"
+            res += comma
+            comma = ', '
+            res += apname
+        res += '\}$'
+        return res
+
+
 def automata(*sources, timeout=None, ignore_abort=True,
              trust_hoa=True, no_sid=False, debug=False):
     """Read automata from a list of sources.
@@ -674,6 +712,19 @@ def translate(formula, *args, dict=_bdd_dict):
 formula.translate = translate
 
 
+# Wrap C++-functions into lambdas so that they get converted into
+# instance methods (i.e., self passed as first argument
+# automatically), because only user-defined functions are converted as
+# instance methods.
+def _add_formula(meth, name = None):
+    setattr(formula, name or meth, (lambda self, *args, **kwargs:
+                                    globals()[meth](self, *args, **kwargs)))
+
+
+_add_formula('contains')
+_add_formula('are_equivalent', 'equivalent_to')
+
+
 def postprocess(automaton, *args, formula=None):
     """Post process an automaton.
 
@@ -712,15 +763,17 @@ twa.postprocess = postprocess
 
 # Wrap C++-functions into lambdas so that they get converted into
 # instance methods (i.e., self passed as first argument
-# automatically), because only used-defined functions are converted as
+# automatically), because only user-defined functions are converted as
 # instance methods.
-def _add_twa_graph(meth):
-    setattr(twa_graph, meth, (lambda self, *args, **kwargs:
-                              globals()[meth](self, *args, **kwargs)))
+def _add_twa_graph(meth, name = None):
+    setattr(twa_graph, name or meth, (lambda self, *args, **kwargs:
+                                      globals()[meth](self, *args, **kwargs)))
 
 for meth in ('scc_filter', 'scc_filter_states',
-             'is_deterministic', 'is_unambiguous'):
+             'is_deterministic', 'is_unambiguous',
+             'contains'):
     _add_twa_graph(meth)
+_add_twa_graph('are_equivalent', 'equivalent_to')
 
 # Wrapper around a formula iterator to which we add some methods of formula
 # (using _addfilter and _addmap), so that we can write things like
@@ -798,10 +851,10 @@ def randltl(ap, n=-1, **kwargs):
     """
     opts = option_map()
     output_map = {
-        "ltl": OUTPUTLTL,
-        "psl": OUTPUTPSL,
-        "bool": OUTPUTBOOL,
-        "sere": OUTPUTSERE
+        "ltl": randltlgenerator.LTL,
+        "psl": randltlgenerator.PSL,
+        "bool": randltlgenerator.Bool,
+        "sere": randltlgenerator.SERE
     }
 
     if isinstance(ap, list):
@@ -836,18 +889,18 @@ def randltl(ap, n=-1, **kwargs):
     dump_priorities = kwargs.get("dump_priorities", False)
     if dump_priorities:
         dumpstream = ostringstream()
-        if output == OUTPUTLTL:
+        if output == randltlgenerator.LTL:
             print('Use argument ltl_priorities=STRING to set the following '
                   'LTL priorities:\n')
             rg.dump_ltl_priorities(dumpstream)
             print(dumpstream.str())
-        elif output == OUTPUTBOOL:
+        elif output == randltlgenerator.Bool:
             print('Use argument boolean_priorities=STRING to set the '
                   'following Boolean formula priorities:\n')
             rg.dump_bool_priorities(dumpstream)
             print(dumpstream.str())
-        elif output == OUTPUTPSL or output == OUTPUTSERE:
-            if output != OUTPUTSERE:
+        elif output == randltlgenerator.PSL or output == randltlgenerator.SERE:
+            if output != randltlgenerator.SERE:
                 print('Use argument ltl_priorities=STRING to set the '
                       'following LTL priorities:\n')
                 rg.dump_psl_priorities(dumpstream)
@@ -880,7 +933,7 @@ def randltl(ap, n=-1, **kwargs):
             if f is None:
                 sys.stderr.write("Warning: could not generate a new "
                                  "unique formula after {} trials.\n"
-                                 .format(MAX_TRIALS))
+                                 .format(randltlgenerator.MAX_TRIALS))
                 raise StopIteration
             self.i += 1
             return f
@@ -931,7 +984,8 @@ for fun in ['remove_x', 'relabel', 'relabel_bse',
 def sat_minimize(aut, acc=None, colored=False,
                  state_based=False, states=0,
                  max_states=0, sat_naive=False, sat_langmap=False,
-                 sat_incr=0, sat_incr_steps=0):
+                 sat_incr=0, sat_incr_steps=0,
+                 display_log=False, return_log=False):
     args=''
     if acc is not None:
         if type(acc) is not str:
@@ -954,9 +1008,25 @@ def sat_minimize(aut, acc=None, colored=False,
     if sat_incr:
         args += ',sat-incr=' + str(sat_incr)
         args += ',sat-incr-steps=' + str(sat_incr_steps)
-
     from spot.impl import sat_minimize as sm
-    return sm(aut, args, state_based)
+
+    if display_log or return_log:
+        import pandas as pd
+        with tempfile.NamedTemporaryFile(dir='.', suffix='.satlog') as t:
+            args += ',log="{}"'.format(t.name)
+            aut = sm(aut, args, state_based)
+            dfrm = pd.read_csv(t.name, dtype=object)
+            if display_log:
+                # old versions of ipython do not import display by default
+                from IPython.display import display
+                del dfrm['automaton']
+                display(dfrm)
+            if return_log:
+                return aut, dfrm
+            else:
+                return aut
+    else:
+        return sm(aut, args, state_based)
 
 
 def parse_word(word, dic=_bdd_dict):
@@ -969,9 +1039,10 @@ def bdd_to_formula(b, dic=_bdd_dict):
 
 def language_containment_checker(dic=_bdd_dict):
     from spot.impl import language_containment_checker as c
+    c.contains = lambda this, a, b: c.contained(this, b, a)
+    c.are_equivalent = c.equal
     return c(dic)
 
-
 def mp_hierarchy_svg(cl=None):
     """
     Return an some string containing an SVG picture of the Manna &
