@@ -17,6 +17,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#include "config.h"
 #include <cmath>
 
 #include <spot/gen/formulas.hh>
@@ -42,19 +43,14 @@ namespace spot
     namespace
     {
       static formula
-      ms_example(const char* a, const char* b, int n)
+      ms_example(const char* a, const char* b, int n, int m)
       {
         formula ax = formula::tt();
         formula fb = formula::tt();
         for (int i = n; i > 0; --i)
-          {
-            std::ostringstream ans;
-            ans << a << i;
-            ax = And_(formula::ap(ans.str()), X_(ax));
-            ans.str("");
-            ans << b << i;
-            fb = F_(And_(formula::ap(ans.str()), fb));
-          }
+          ax = And_(formula::ap(a + std::to_string(i)), X_(ax));
+        for (int i = m; i > 0; --i)
+          fb = F_(And_(formula::ap(b + std::to_string(i)), fb));
         return And_(G_(F_(ax)), fb);
       }
 
@@ -260,9 +256,20 @@ namespace spot
         return result;
       }
 
+      // Builds X(X(...X(p))) with n occurrences of X.
       static formula
-      LTL_GF_equiv_implies(int n, const std::string& a, const std::string& z,
-                           bool equiv)
+      X_n(formula p, int n)
+      {
+        assert(n >= 0);
+        formula res = p;
+        while (n--)
+          res = X_(res);
+        return res;
+      }
+
+      static formula
+      GF_equiv_implies(int n, const std::string& a, const std::string& z,
+                       bool equiv)
       {
         formula left = GF_n(a, n);
         formula right = formula::G(formula::F(formula::ap(z)));
@@ -272,6 +279,19 @@ namespace spot
           return formula::Implies(left, right);
       }
 
+      static formula
+      GF_equiv_implies_xn(int n, const std::string& a, bool equiv)
+      {
+        formula ap = formula::ap(a);
+        formula xn = X_n(ap, n);
+        formula in;
+        if (equiv)
+          in = formula::Equiv(ap, xn);
+        else
+          in = formula::Implies(ap, xn);
+        return G_(F_(in));
+      }
+
       //  (((p1 OP p2) OP p3)...OP pn)   if right_assoc == false
       //  (p1 OP (p2 OP (p3 OP (... pn)  if right_assoc == true
       static formula
@@ -406,17 +426,6 @@ namespace spot
         return Not_(Implies_(fair, resp));
       }
 
-      // Builds X(X(...X(p))) with n occurrences of X.
-      static formula
-      X_n(formula p, int n)
-      {
-        assert(n >= 0);
-        formula res = p;
-        while (n--)
-          res = X_(res);
-        return res;
-      }
-
       // Based on LTLcounter.pl from Kristin Rozier.
       // http://shemesh.larc.nasa.gov/people/kyr/benchmarking_scripts/
       static formula
@@ -1010,7 +1019,7 @@ namespace spot
         int k = ceil(log2(n)) + (n == 1);
 
         if (n <= 0)
-          bad_number("kr1-exp", n);
+          bad_number("kr-nlogn", n);
 
         formula fa = formula::ap(a);
         formula fb = formula::ap(b);
@@ -1138,9 +1147,58 @@ namespace spot
 
         return res;
       }
+
+      static formula
+      sejk_f(std::string a, std::string b, int n, int m)
+      {
+        formula left = G_(F_(formula::ap(a + std::to_string(0))));
+        formula right = X_n(formula::ap(b), m);
+        formula f0 = U_(left, right);
+        for (int i = 1; i <= n; ++i)
+          {
+            formula left = G_(F_(formula::ap(a + std::to_string(i))));
+            f0 = U_(left, G_(f0));
+          }
+        return f0;
+      }
+
+      static formula
+      sejk_j(std::string a, std::string b, int n)
+      {
+        return formula::Implies(GF_n(a, n), GF_n(b, n));
+      }
+
+      static formula
+      sejk_k(std::string a, std::string b, int n)
+      {
+        formula result = formula::tt();
+        for (int i = 1; i <= n; ++i)
+          {
+            formula ai = formula::ap(a + std::to_string(i));
+            formula bi = formula::ap(b + std::to_string(i));
+            result = formula::And({result,
+                                   formula::Or({G_(F_(ai)), F_(G_(bi))})});
+          }
+        return result;
+      }
+
+      static formula
+      sejk_pattern(int n)
+      {
+        static const char* formulas[] = {
+          "GF(Fa | Gb | FG(a | Xb))",
+          "FG(Ga | F!b | GF(a & Xb))",
+          "GF(Fa | GXb | FG(a | XXb))",
+        };
+
+        constexpr unsigned max = (sizeof formulas)/(sizeof *formulas);
+        if (n < 1 || (unsigned) n > max)
+          bad_number("sejk-patterns", n, max);
+        return spot::relabel(parse_formula(formulas[n - 1]), Pnn);
+      }
     }
 
-    formula ltl_pattern(ltl_pattern_id pattern, int n)
+    formula ltl_pattern(ltl_pattern_id pattern, int n, int m)
     {
       if (n < 0)
         {
@@ -1149,6 +1207,13 @@ namespace spot
               << " should be positive";
           throw std::runtime_error(err.str());
         }
+      if ((m >= 0) ^ (ltl_pattern_argc(pattern) == 2))
+        {
+          std::ostringstream err;
+          err << "unexpected number of arguments for "
+              << ltl_pattern_name(pattern);
+          throw std::runtime_error(err.str());
+        }
 
       switch (pattern)
         {
@@ -1172,9 +1237,13 @@ namespace spot
         case LTL_FXG_OR:
           return FXG_or_n("p", n);
         case LTL_GF_EQUIV:
-          return LTL_GF_equiv_implies(n, "a", "z", true);
+          return GF_equiv_implies(n, "a", "z", true);
+        case LTL_GF_EQUIV_XN:
+          return GF_equiv_implies_xn(n, "a", true);
         case LTL_GF_IMPLIES:
-          return LTL_GF_equiv_implies(n, "a", "z", false);
+          return GF_equiv_implies(n, "a", "z", false);
+        case LTL_GF_IMPLIES_XN:
+          return GF_equiv_implies_xn(n, "a", false);
         case LTL_GH_Q:
           return Q_n("p", n);
         case LTL_GH_R:
@@ -1198,7 +1267,7 @@ namespace spot
         case LTL_OR_GF:
           return GF_n("p", n, false);
         case LTL_MS_EXAMPLE:
-          return ms_example("a", "b", n);
+          return ms_example("a", "b", n, m);
         case LTL_MS_PHI_H:
           return ms_phi_h("a", "b", n);
         case LTL_MS_PHI_R:
@@ -1221,6 +1290,14 @@ namespace spot
           return ltl_counter("b", "m", n, true);
         case LTL_SB_PATTERNS:
           return sb_pattern(n);
+        case LTL_SEJK_F:
+          return sejk_f("a", "b", n, m);
+        case LTL_SEJK_J:
+          return sejk_j("a", "b", n);
+        case LTL_SEJK_K:
+          return sejk_k("a", "b", n);
+        case LTL_SEJK_PATTERNS:
+          return sejk_pattern(n);
         case LTL_TV_F1:
           return tv_f1("p", "q", n);
         case LTL_TV_F2:
@@ -1256,7 +1333,9 @@ namespace spot
           "eh-patterns",
           "fxg-or",
           "gf-equiv",
+          "gf-equiv-xn",
           "gf-implies",
+          "gf-implies-xn",
           "gh-q",
           "gh-r",
           "go-theta",
@@ -1280,6 +1359,10 @@ namespace spot
           "rv-counter-carry-linear",
           "rv-counter-linear",
           "sb-patterns",
+          "sejk-f",
+          "sejk-j",
+          "sejk-k",
+          "sejk-patterns",
           "tv-f1",
           "tv-f2",
           "tv-g1",
@@ -1315,7 +1398,9 @@ namespace spot
           return 12;
         case LTL_FXG_OR:
         case LTL_GF_EQUIV:
+        case LTL_GF_EQUIV_XN:
         case LTL_GF_IMPLIES:
+        case LTL_GF_IMPLIES_XN:
         case LTL_GH_Q:
         case LTL_GH_R:
         case LTL_GO_THETA:
@@ -1345,6 +1430,12 @@ namespace spot
           return 0;
         case LTL_SB_PATTERNS:
           return 27;
+        case LTL_SEJK_F:
+        case LTL_SEJK_J:
+        case LTL_SEJK_K:
+          return 0;
+        case LTL_SEJK_PATTERNS:
+          return 3;
         case LTL_TV_F1:
         case LTL_TV_F2:
         case LTL_TV_G1:
@@ -1357,8 +1448,69 @@ namespace spot
           break;
         }
       throw std::runtime_error("unsupported pattern");
-
     }
 
+    int ltl_pattern_argc(ltl_pattern_id pattern)
+    {
+      switch (pattern)
+        {
+          // Keep this alphabetically-ordered!
+        case LTL_AND_F:
+        case LTL_AND_FG:
+        case LTL_AND_GF:
+        case LTL_CCJ_ALPHA:
+        case LTL_CCJ_BETA:
+        case LTL_CCJ_BETA_PRIME:
+        case LTL_DAC_PATTERNS:
+        case LTL_EH_PATTERNS:
+        case LTL_FXG_OR:
+        case LTL_GF_EQUIV:
+        case LTL_GF_EQUIV_XN:
+        case LTL_GF_IMPLIES:
+        case LTL_GF_IMPLIES_XN:
+        case LTL_GH_Q:
+        case LTL_GH_R:
+        case LTL_GO_THETA:
+        case LTL_GXF_AND:
+        case LTL_HKRSS_PATTERNS:
+        case LTL_KR_N:
+        case LTL_KR_NLOGN:
+        case LTL_KV_PSI:
+          return 1;
+        case LTL_MS_EXAMPLE:
+          return 2;
+        case LTL_MS_PHI_H:
+        case LTL_MS_PHI_R:
+        case LTL_MS_PHI_S:
+        case LTL_OR_FG:
+        case LTL_OR_G:
+        case LTL_OR_GF:
+        case LTL_P_PATTERNS:
+        case LTL_R_LEFT:
+        case LTL_R_RIGHT:
+        case LTL_RV_COUNTER_CARRY:
+        case LTL_RV_COUNTER_CARRY_LINEAR:
+        case LTL_RV_COUNTER:
+        case LTL_RV_COUNTER_LINEAR:
+        case LTL_SB_PATTERNS:
+          return 1;
+        case LTL_SEJK_F:
+          return 2;
+        case LTL_SEJK_J:
+        case LTL_SEJK_K:
+        case LTL_SEJK_PATTERNS:
+        case LTL_TV_F1:
+        case LTL_TV_F2:
+        case LTL_TV_G1:
+        case LTL_TV_G2:
+        case LTL_TV_UU:
+        case LTL_U_LEFT:
+        case LTL_U_RIGHT:
+          return 1;
+        case LTL_END:
+          break;
+        }
+      throw std::runtime_error("unsupported pattern");
+    }
   }
 }
