@@ -17,6 +17,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#include "config.h"
 #include <ltdl.h>
 #include <cstring>
 #include <cstdlib>
@@ -108,7 +109,7 @@ namespace spot
       {
         if (--count)
           return;
-        pool->deallocate(this);
+        pool->deallocate(const_cast<spins_state*>(this));
       }
 
       size_t hash() const override
@@ -233,7 +234,7 @@ namespace spot
       fixed_size_pool* p = static_cast<fixed_size_pool*>(ctx->pool);
       spins_state* out =
         new(p->allocate()) spins_state(ctx->state_size, p);
-      assert(out != nullptr);
+      SPOT_ASSUME(out != nullptr);
       memcpy(out->vars, dst, ctx->state_size * sizeof(int));
       out->compute_hash();
       ctx->transitions.emplace_back(out);
@@ -251,7 +252,7 @@ namespace spot
                               - sizeof(spins_compressed_state::vars)
                               + sizeof(int) * csize);
       spins_compressed_state* out = new(mem) spins_compressed_state(csize, p);
-      assert(out != nullptr);
+      SPOT_ASSUME(out != nullptr);
       memcpy(out->vars, ctx->compressed, csize * sizeof(int));
       out->compute_hash();
       ctx->transitions.emplace_back(out);
@@ -712,7 +713,7 @@ namespace spot
                                     + sizeof(int) * csize);
             spins_compressed_state* res = new(mem)
               spins_compressed_state(csize, p);
-            assert(res != nullptr);
+            SPOT_ASSUME(res != nullptr);
             memcpy(res->vars, compressed_, csize * sizeof(int));
             res->compute_hash();
             return res;
@@ -721,7 +722,7 @@ namespace spot
           {
             fixed_size_pool* p = const_cast<fixed_size_pool*>(&statepool_);
             spins_state* res = new(p->allocate()) spins_state(state_size_, p);
-            assert(res != nullptr);
+            SPOT_ASSUME(res != nullptr);
             d_->get_initial_state(res->vars);
             res->compute_hash();
             return res;
