@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2010-2017 Laboratoire de Recherche et Développement de
+// Copyright (C) 2010-2018 Laboratoire de Recherche et Développement de
 // l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -19,6 +19,7 @@
 
 //#define TRACE
 
+#include "config.h"
 #include <iostream>
 #ifdef TRACE
 #define trace std::clog
@@ -28,7 +29,6 @@
 
 #include <spot/twa/formula2bdd.hh>
 #include <cassert>
-#include <iostream>
 #include <spot/twa/bddprint.hh>
 #include <stack>
 #include <spot/taalgos/tgba2ta.hh>
@@ -174,7 +174,7 @@ namespace spot
               }
 
             sscc.push(++num);
-            arc.push(0U);
+            arc.push({});
             sscc.top().is_accepting
               = testing_aut->is_accepting_state(init);
             twa_succ_iterator* iter = testing_aut->succ_iter(init);
@@ -406,7 +406,7 @@ namespace spot
           twa_succ_iterator* it = tgba_->succ_iter(tgba_init_state);
           it->first();
           if (!it->done())
-            is_acc = it->acc() != 0U;
+            is_acc = !!it->acc();
           delete it;
         }
 
@@ -458,7 +458,7 @@ namespace spot
                     twa_succ_iterator* it = tgba_->succ_iter(tgba_state);
                     it->first();
                     if (!it->done())
-                      is_acc = it->acc() != 0U;
+                      is_acc = !!it->acc();
                     delete it;
                   }
 
@@ -630,7 +630,7 @@ namespace spot
 
         if (state->compare(ta->get_artificial_initial_state()))
           ta->create_transition(state, bdd_stutering_transition,
-                                0U, state);
+                                {}, state);
 
         state->set_livelock_accepting_state(false);
         state->set_accepting_state(false);
