@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2015, 2016 Laboratoire de Recherche et Développement de
+// Copyright (C) 2015, 2016, 2017 Laboratoire de Recherche et Développement de
 // l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -105,7 +105,7 @@ namespace spot
         {
           if ((*i)->is_boolean())
             {
-              b.push_back(*i);
+              b.emplace_back(*i);
               i = v.erase(i);
             }
           else
@@ -124,7 +124,7 @@ namespace spot
     }
   }
 
-  const fnode* fnode::unique(const fnode* f)
+  const fnode* fnode::unique(fnode* f)
   {
     auto ires = m.uniq.emplace(f);
     if (!ires.second)
@@ -132,7 +132,8 @@ namespace spot
         //(*ires.first)->dump(std::cerr << "UNI: ") << '\n';
         for (auto c: *f)
           c->destroy();
-        delete f;
+        f->~fnode();
+        ::operator delete(f);
         return (*ires.first)->clone();
       }
     //f->dump(std::cerr << "INS: ") << '\n';
@@ -158,7 +159,48 @@ namespace spot
         for (auto c: *this)
           c->destroy();
       }
-    delete this;
+    this->~fnode();
+    ::operator delete(const_cast<fnode*>(this));
+  }
+
+  void
+  fnode::report_non_existing_child()
+  {
+    throw std::runtime_error("access to non-existing child");
+  }
+
+  void
+  fnode::report_too_many_children()
+  {
+    throw std::runtime_error("too many children for formula");
+  }
+
+  void
+  fnode::report_get_child_of_expecting_single_child_node()
+  {
+    throw std::invalid_argument
+      ("get_child_of() expecting single-child node");
+  }
+
+  void
+  fnode::report_min_invalid_arg()
+  {
+    throw std::invalid_argument
+      ("min() only works on Star and FStar nodes");
+  }
+
+  void
+  fnode::report_max_invalid_arg()
+  {
+    throw std::invalid_argument
+      ("min() only works on Star and FStar nodes");
+  }
+
+  void
+  formula::report_ap_invalid_arg()
+  {
+    throw std::invalid_argument("atomic propositions cannot be "
+                                "constructed from arbitrary formulas");
   }
 
   std::string fnode::kindstr() const
@@ -230,7 +272,7 @@ namespace spot
             {
               unsigned ps = (*i)->size();
               for (unsigned n = 0; n < ps; ++n)
-                inlined.push_back((*i)->nth(n)->clone());
+                inlined.emplace_back((*i)->nth(n)->clone());
               (*i)->destroy();
               // FIXME: Do not use erase.  See previous FIXME.
               i = v.erase(i);
@@ -242,7 +284,7 @@ namespace spot
           // For concat we have to keep track of the order of
           // all the arguments.
           if (o == op::Concat || o == op::Fusion)
-            inlined.push_back(*i);
+            inlined.emplace_back(*i);
           ++i;
         }
       if (o == op::Concat || o == op::Fusion)
@@ -470,10 +512,10 @@ namespace spot
                       i->destroy();
                       continue;
                     }
-                  tmp.push_back(i);
+                  tmp.emplace_back(i);
                 }
               if (tmp.empty())
-                tmp.push_back(weak_abs);
+                tmp.emplace_back(weak_abs);
               v.swap(tmp);
             }
         }
@@ -575,9 +617,8 @@ namespace spot
         v.insert(v.begin(), tt());
       }
 
-
-    auto mem = operator new(sizeof(fnode)
-                            + (v.size() - 1)*sizeof(*children));
+    auto mem = ::operator new(sizeof(fnode)
+                              + (v.size() - 1)*sizeof(*children));
     return unique(new(mem) fnode(o, v.begin(), v.end()));
   }
 
@@ -989,7 +1030,7 @@ namespace spot
         SPOT_UNREACHABLE();
       }
 
-    auto mem = operator new(sizeof(fnode) + sizeof(*children));
+    auto mem = ::operator new(sizeof(fnode) + sizeof(*children));
     return unique(new(mem) fnode(o, {first, second}));
   }
 
@@ -1025,16 +1066,15 @@ namespace spot
 
   void fnode::setup_props(op o)
   {
-    op_ = o;
     id_ = next_id_++;
     // If the counter of formulae ever loops, we want to skip the
     // first three values, because they are permanently associated
     // to constants, and it is convenient to have constants
     // smaller than all other formulas.
-    if (next_id_ == 0)
+    if (SPOT_UNLIKELY(next_id_ == 0))
       next_id_ = 3;
 
-    switch (op_)
+    switch (o)
       {
       case op::ff:
       case op::tt:
@@ -1242,7 +1282,8 @@ namespace spot
         props = children[0]->props & children[1]->props;
         is_.eventual = false;
         is_.universal = false;
-        is_.sere_formula = is_.boolean;
+        is_.sere_formula = (children[0]->is_boolean()
+                            && children[1]->is_sere_formula());
         is_.sugar_free_boolean = false;
         is_.in_nenoform = false;
         is_.syntactic_safety = (children[0]->is_syntactic_guarantee()
@@ -1277,8 +1318,9 @@ namespace spot
         else
           {
             is_.syntactic_safety = false;
-            is_.syntactic_obligation = children[1]->is_syntactic_guarantee();
-            is_.syntactic_recurrence = children[1]->is_syntactic_guarantee();
+            bool g = children[1]->is_syntactic_guarantee();
+            is_.syntactic_obligation = g;
+            is_.syntactic_recurrence = g;
           }
         assert(children[0]->is_sere_formula());
         assert(children[1]->is_psl_formula());
@@ -1306,8 +1348,9 @@ namespace spot
         else
           {
             is_.syntactic_guarantee = false;
-            is_.syntactic_obligation = children[1]->is_syntactic_safety();
-            is_.syntactic_persistence = children[1]->is_syntactic_safety();
+            bool s = children[1]->is_syntactic_safety();
+            is_.syntactic_obligation = s;
+            is_.syntactic_persistence = s;
           }
         assert(children[0]->is_sere_formula());
         assert(children[1]->is_psl_formula());
@@ -1623,7 +1666,7 @@ namespace spot
           v.reserve(s - 1);
           for (unsigned j = 0; j < s; ++j)
             if (i != j)
-              v.push_back(nth(j)->clone());
+              v.emplace_back(nth(j)->clone());
           return multop(o, v);
         }
       default:
