@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2011, 2014-2017 Laboratoire de Recherche et Developpement de
+// Copyright (C) 2011, 2014-2018 Laboratoire de Recherche et Developpement de
 // l'EPITA (LRDE).
 // Copyright (C) 2003, 2004, 2005 Laboratoire d'Informatique de Paris 6 (LIP6),
 // département Systèmes Répartis Coopératifs (SRC), Université Pierre
@@ -20,6 +20,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#include "config.h"
 #include <spot/twa/twa.hh>
 #include <spot/twa/twagraph.hh>
 #include <spot/twaalgos/couvreurnew.hh>
@@ -27,6 +28,9 @@
 #include <spot/twaalgos/remfin.hh>
 #include <spot/twaalgos/alternation.hh>
 #include <spot/twa/twaproduct.hh>
+#include <spot/twaalgos/dualize.hh>
+#include <spot/twaalgos/postproc.hh>
+#include <spot/twaalgos/isdet.hh>
 #include <utility>
 
 namespace spot
@@ -81,7 +85,7 @@ namespace spot
   {
     if (acc().uses_fin_acceptance())
       throw std::runtime_error("twa::accepting_run() does not work with "
-                               "Fin acceptance (but twa:is_empty() and "
+                               "Fin acceptance (but twa::is_empty() and "
                                "twa::accepting_word() can)");
     auto res = couvreur99_new_check(shared_from_this());
     if (!res)
@@ -116,7 +120,7 @@ namespace spot
   twa::intersecting_run(const_twa_ptr other, bool from_other) const
   {
     auto a = shared_from_this();
-    if (from_other)
+    if (!from_other)
       other = remove_fin_maybe(other);
     else
       a = remove_fin_maybe(a);
@@ -134,6 +138,57 @@ namespace spot
     return otf_product(a1, a2)->accepting_word();
   }
 
+  namespace
+  {
+    static const_twa_graph_ptr
+    ensure_deterministic(const const_twa_ptr& aut_in)
+    {
+      const_twa_graph_ptr aut =
+        std::dynamic_pointer_cast<const twa_graph>(aut_in);
+      if (!aut)
+        aut = make_twa_graph(aut_in, twa::prop_set::all());
+
+      if (is_deterministic(aut))
+        return aut;
+      postprocessor p;
+      p.set_type(postprocessor::Generic);
+      p.set_pref(postprocessor::Deterministic);
+      p.set_level(postprocessor::Low);
+      return p.run(std::const_pointer_cast<twa_graph>(aut));
+    }
+  }
+  twa_run_ptr
+  twa::exclusive_run(const_twa_ptr other) const
+  {
+    const_twa_ptr a = shared_from_this();
+    const_twa_ptr b = other;
+
+    // We have to find a run in A\B or in B\A.  When possible, let's
+    // make sure the first automaton we complement is deterministic.
+    if (auto aa = std::dynamic_pointer_cast<const twa_graph>(a))
+      if (is_deterministic(aa))
+        std::swap(a, b);
+    if (auto run = a->intersecting_run(dualize(ensure_deterministic(b))))
+      return run;
+    return b->intersecting_run(dualize(ensure_deterministic(a)));
+  }
+
+  twa_word_ptr
+  twa::exclusive_word(const_twa_ptr other) const
+  {
+    const_twa_ptr a = shared_from_this();
+    const_twa_ptr b = other;
+
+    // We have to find a word in A\B or in B\A.  When possible, let's
+    // make sure the first automaton we complement is deterministic.
+    if (auto aa = std::dynamic_pointer_cast<const twa_graph>(a))
+      if (is_deterministic(aa))
+        std::swap(a, b);
+    if (auto word = a->intersecting_word(dualize(ensure_deterministic(b))))
+      return word;
+    return b->intersecting_word(dualize(ensure_deterministic(a)));
+  }
+
   void
   twa::set_named_prop(std::string s, std::nullptr_t)
   {
