@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2016, 2017 Laboratoire de Recherche et Développement
+// Copyright (C) 2016-2018 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -17,6 +17,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#include "config.h"
 #include <algorithm>
 #include <sstream>
 #include <spot/twaalgos/alternation.hh>
@@ -108,7 +109,7 @@ namespace spot
       {
         bool first = true;
         bool reject_cycle = false;
-        acc_cond::mark_t m = 0U;
+        acc_cond::mark_t m = {};
         for (unsigned src: si_.states_of(scc))
           for (auto& t: aut_->out(src))
             for (unsigned d: aut_->univ_dests(t.dst))
@@ -157,8 +158,9 @@ namespace spot
                     assert(si_.is_accepting_scc(n));
 
                     // Catch unsupported types of automata
-                    bool rej = ensure_weak_scc(n);
-                    assert(rej == false);
+                    bool rejecting = ensure_weak_scc(n);
+                    assert(!rejecting);
+                    (void) rejecting;
                     // Detect if it is a "true state"
                     unsigned s = si_.states_of(n).front();
                     auto& ss = g.state_storage(s);
@@ -182,6 +184,7 @@ namespace spot
       std::map<int, unsigned> var_to_state_;
       std::vector<int> scc_to_var_;
       std::map<int, acc_cond::mark_t> var_to_mark_;
+      std::vector<unsigned> mark_to_state_;
       bdd all_vars_;
       bdd all_marks_;
       bdd all_states_;
@@ -221,6 +224,7 @@ namespace spot
         unsigned nc = si_.scc_count();
         scc_to_var_.reserve(nc);
         unsigned mark_pos = has_reject_more_;
+        mark_to_state_.resize(mark_pos);
         bdd all_marks = bddtrue;
         for (unsigned s = 0; s < nc; ++s)
           {
@@ -229,6 +233,7 @@ namespace spot
               {
                 int v = d->register_anonymous_variables(1, this);
                 scc_to_var_.emplace_back(v);
+                mark_to_state_.push_back(si_.one_state_of(c));
                 var_to_mark_.emplace(v, acc_cond::mark_t({mark_pos++}));
                 bdd bv = bdd_ithvar(v);
                 all_marks &= bv;
@@ -268,7 +273,7 @@ namespace spot
                   marked && (scc_s == scc_d) && (c == scc_class::reject_more);
                 dest &= bdd_ithvar(state_to_var_[d] + mark);
                 if (c == scc_class::reject_1 && scc_s == scc_d)
-                  dest &= bdd_ithvar(scc_to_var_[scc_s]);
+                  dest &= bdd_ithvar(scc_to_var_[scc_d]);
               }
             res |= e.cond & dest;
           }
@@ -279,7 +284,7 @@ namespace spot
 
       acc_cond::mark_t bdd_to_state(bdd b, std::vector<unsigned>& s)
       {
-        acc_cond::mark_t m = 0U;
+        acc_cond::mark_t m = {};
         while (b != bddtrue)
           {
             assert(bdd_low(b) == bddfalse);
@@ -462,6 +467,29 @@ namespace spot
                     bdd dest = bdd_existcomp(cube, all_vars_);
                     v.clear();
                     acc_cond::mark_t m = bdd_to_state(dest, v);
+
+                    // if there is no promise "f" is between a state
+                    // that does not have f, and a state that have
+                    // "f", we can add one.  Doing so will help later
+                    // simplifications performed by postprocessor.  An
+                    // example where this is needed is the VWAA
+                    // generated by ltl[23]ba for GFa.  Without the
+                    // next loop, the final TGBA has 2 states instead
+                    // of 1.
+                    for (unsigned m1: (all_marks - m).sets())
+                      {
+                        if (has_reject_more_ && m1 == 0)
+                          continue;
+                        auto& sv = s_to_ss[s];
+                        unsigned ms = mark_to_state_[m1];
+                        if (std::find(v.begin(), v.end(), ms) != v.end())
+                          {
+                            unsigned ms = mark_to_state_[m1];
+                            if (std::find(sv.begin(), sv.end(), ms) == sv.end())
+                              m.set(m1);
+                          }
+                      }
+
                     unsigned d = new_state(v, has_mark);
                     if (has_mark)
                       m.set(0);
@@ -648,7 +676,9 @@ namespace spot
     virtual acc_cond::mark_t acc() const override
     {
       assert(dst_);
-      return dst_->is_reset();
+      return dst_->is_reset() ?
+                acc_cond::mark_t({0}) :
+                acc_cond::mark_t({});
     }
   };
 
