@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2011, 2012, 2014-2017 Laboratoire de Recherche
+// Copyright (C) 2011, 2012, 2014-2018 Laboratoire de Recherche
 // et Développement de l'Epita (LRDE).
 // Copyright (C) 2003, 2004  Laboratoire d'Informatique de Paris 6 (LIP6),
 // département Systèmes Répartis Coopératifs (SRC), Université Pierre
@@ -20,6 +20,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#include "config.h"
 #include <ostream>
 #include <sstream>
 #include <stdexcept>
@@ -27,9 +28,7 @@
 #include <spot/twa/twagraph.hh>
 #include <spot/twaalgos/dot.hh>
 #include <spot/twa/bddprint.hh>
-#include <spot/twaalgos/reachiter.hh>
 #include <spot/misc/escape.hh>
-#include <spot/twa/twagraph.hh>
 #include <spot/twa/formula2bdd.hh>
 #include <spot/twaalgos/sccinfo.hh>
 #include <spot/kripke/fairkripke.hh>
@@ -74,32 +73,37 @@ namespace spot
       std::vector<std::string>* sn_ = nullptr;
       std::map<unsigned, unsigned>* highlight_edges_ = nullptr;
       std::map<unsigned, unsigned>* highlight_states_ = nullptr;
+      bool highlight_states_show_num_ = false;
       std::vector<std::pair<unsigned, unsigned>>* sprod_ = nullptr;
       std::vector<unsigned>* orig_ = nullptr;
       std::set<unsigned>* incomplete_ = nullptr;
-      std::string* name_ = nullptr;
+      std::string* name_ = nullptr; // title for the graph
+      std::string* graph_name_ = nullptr; // name for the digraph
       std::map<std::pair<int, int>, int> univ_done;
+      std::vector<bool> true_states_;
 
-      acc_cond::mark_t inf_sets_ = 0U;
-      acc_cond::mark_t fin_sets_ = 0U;
+      acc_cond::mark_t inf_sets_ = {};
+      acc_cond::mark_t fin_sets_ = {};
       unsigned opt_shift_sets_ = 0;
       enum { ShapeAuto = 0, ShapeCircle, ShapeEllipse } opt_shape_ = ShapeAuto;
       bool opt_force_acc_trans_ = false;
       bool opt_vertical_ = false;
       bool opt_name_ = false;
-      bool opt_show_acc_ = false;
+      bool opt_show_acc_ = true;
       bool mark_states_ = false;
       bool dcircles_ = false;
       bool opt_scc_ = false;
       bool opt_html_labels_ = false;
       bool opt_color_sets_ = false;
       bool opt_state_labels_ = false;
+      bool uppercase_k_seen_ = false;
       bool opt_rainbow = false;
       bool opt_bullet = false;
       bool opt_bullet_but_buchi = false;
       bool opt_all_bullets = false;
       bool opt_ordered_edges_ = false;
       bool opt_numbered_edges_ = false;
+      bool opt_hide_true_states_ = false;
       bool opt_orig_show_ = false;
       bool max_states_given_ = false; // related to max_states_
       bool opt_latex_ = false;
@@ -111,7 +115,7 @@ namespace spot
       std::string opt_font_;
       std::string opt_node_color_;
       std::ostream& os_;
-      bool opt_want_state_names_ = true;
+      bool inline_state_names_ = true;
       unsigned max_states_ = -1U; // related to max_states_given_
 
       bool opt_shared_univ_dest_ = true;
@@ -128,6 +132,28 @@ namespace spot
         return max_states_given_;
       }
 
+      void find_true_states()
+      {
+        unsigned n = aut_->num_states();
+        true_states_.resize(n, false);
+        auto& g = aut_->get_graph();
+        auto& acc = aut_->acc();
+        for (unsigned s = 0; s < n; ++s)
+          {
+            auto& ss = g.state_storage(s);
+            if (ss.succ == ss.succ_tail)
+              {
+                auto& es = g.edge_storage(ss.succ);
+                if (es.cond == bddtrue
+
+                    && es.src == es.dst
+                    && acc.accepting(es.acc))
+                  true_states_[s] = true;
+              }
+          }
+      }
+
+
       void
       parse_opts(const char* options)
       {
@@ -188,7 +214,7 @@ namespace spot
               opt_numbered_edges_ = true;
               break;
             case '1':
-              opt_want_state_names_ = false;
+              inline_state_names_ = false;
               break;
             case 'a':
               opt_show_acc_ = true;
@@ -245,6 +271,10 @@ namespace spot
             case 'k':
               opt_state_labels_ = true;
               break;
+            case 'K':
+              opt_state_labels_ = false;
+              uppercase_k_seen_ = true;
+              break;
             case 'n':
               opt_name_ = true;
               break;
@@ -265,6 +295,9 @@ namespace spot
             case 's':
               opt_scc_ = true;
               break;
+            case 'u':
+              opt_hide_true_states_ = true;
+              break;
             case 'v':
               opt_vertical_ = true;
               break;
@@ -304,6 +337,11 @@ namespace spot
         parse_opts(options ? options : ".");
       }
 
+      bool uppercase_k_seen() const
+      {
+        return uppercase_k_seen_;
+      }
+
       const char*
       html_set_color(int v) const
       {
@@ -428,6 +466,13 @@ namespace spot
         return tmp_dst.str();
       }
 
+      template<typename U, typename V>
+      void print_true_state(U to, V from) const
+      {
+        os_ << "  T" << to << 'T' << from << " [label=\"\", style=invis, ";
+        os_ << (opt_vertical_ ? "height=0]\n" : "width=0]\n");
+      }
+
       void
       print_dst(int dst, bool print_edges = false,
                 const char* style = nullptr, int color_num = -1)
@@ -442,7 +487,15 @@ namespace spot
           {
             for (unsigned d: aut_->univ_dests(dst))
               {
-                os_ << "  " << dest << " -> " << d;
+                bool dst_is_hidden_true_state =
+                  opt_hide_true_states_ && true_states_[d];
+                if (dst_is_hidden_true_state)
+                  print_true_state(d, dest);
+                os_ << "  " << dest << " -> ";
+                if (dst_is_hidden_true_state)
+                  os_ << 'T' << d << 'T' << dest;
+                else
+                  os_ << d;
                 if (style && *style)
                   os_ << " [" << style << ']';
                 os_ << '\n';
@@ -472,7 +525,10 @@ namespace spot
         // UTF-8 has no glyphs for circled numbers larger than MAX_BULLET.
         if (opt_bullet && (aut_->num_sets() <= MAX_BULLET || opt_latex_))
           opt_all_bullets = true;
-        os_ << "digraph G {\n";
+        os_ << "digraph \"";
+        if (graph_name_)
+          escape_str(os_, *graph_name_);
+        os_ << "\" {\n";
         if (opt_latex_)
           {
             os_ << "  d2tgraphstyle=\"every node/.style={align=center}\"\n";
@@ -497,6 +553,8 @@ namespace spot
           }
         if (!opt_vertical_)
           os_ << "  rankdir=LR\n";
+        if (highlight_states_show_num_)
+          os_ << "  forcelabels=true\n";
         if (name_ || opt_show_acc_)
           {
             os_ << "  " << label_pre_;
@@ -575,22 +633,42 @@ namespace spot
         os_ << '}' << std::endl;
       }
 
+      bool print_state_name(std::ostream& os, unsigned s,
+                            bool force_text = false) const
+      {
+        if (sn_ && s < sn_->size() && !(*sn_)[s].empty())
+          {
+            if (force_text)
+              escape_str(os, (*sn_)[s]);
+            else
+              escape_for_output(os, (*sn_)[s]);
+          }
+        else if (sprod_)
+          {
+            os << (*sprod_)[s].first << ',' << (*sprod_)[s].second;
+          }
+        else
+          {
+            return false;
+          }
+        return true;
+      }
+
       void
       process_state(unsigned s)
       {
         os_ << "  " << s << " [" << label_pre_;
-        if (sn_ && s < sn_->size() && !(*sn_)[s].empty())
-          escape_for_output(os_, (*sn_)[s]);
-        else if (sprod_)
-          os_ << (*sprod_)[s].first << ',' << (*sprod_)[s].second;
-        else
+
+        if (!(inline_state_names_ && print_state_name(os_, s)))
           os_ << s;
         if (orig_ && s < orig_->size())
           os_ << " (" << (*orig_)[s] << ')';
 
+        bool include_state_labels =
+          opt_state_labels_ && (inline_state_names_ || opt_latex_);
         if (mark_states_ && !dcircles_)
           {
-            acc_cond::mark_t acc = 0U;
+            acc_cond::mark_t acc = {};
             for (auto& t: aut_->out(s))
               {
                 acc = t.acc;
@@ -602,13 +680,13 @@ namespace spot
                 os_ << nl_;
                 output_mark(acc);
               }
-            if (opt_state_labels_)
+            if (include_state_labels)
               format_state_label(os_ << nl_, s);
             os_ << label_post_;
           }
         else
           {
-            if (opt_state_labels_)
+            if (include_state_labels)
               format_state_label(os_ << nl_, s);
             os_ << label_post_;
             // Use state_acc_sets(), not state_is_accepting() because
@@ -627,8 +705,25 @@ namespace spot
                   os_ << ",filled";
                 os_ << "\", color=\"" << palette[iter->second % palette_mod]
                     << '"';
+                if (highlight_states_show_num_)
+                  os_ << ", xlabel=\"[" << iter->second << "]\"";
               }
           }
+        if (!inline_state_names_ && (sn_ || sprod_ || opt_state_labels_)
+            && !opt_latex_)
+          {
+            std::ostringstream os;
+            bool nonempty = print_state_name(os, s, true);
+            if (opt_state_labels_)
+              {
+                if (nonempty)
+                  os << '\n';
+                format_state_label(os, s);
+                nonempty = true;
+              }
+            if (nonempty)
+              os_ << ", tooltip=\"" << os.str() << '"';
+          }
         os_ << "]\n";
         if (incomplete_ && incomplete_->find(s) != incomplete_->end())
           os_ << "  u" << s << " [label=\"...\", shape=none, width=0, height=0"
@@ -639,11 +734,20 @@ namespace spot
       process_link(const twa_graph::edge_storage_t& t, int number,
                    bool print_edges)
       {
+        bool dst_is_hidden_true_state =
+          (t.dst < true_states_.size()) && true_states_[t.dst];
+
         if (print_edges)
           {
-            os_ << "  " << t.src << " -> " << (int)t.dst;
+            if (dst_is_hidden_true_state)
+              print_true_state(t.dst, t.src);
+            os_ << "  " << t.src << " -> ";
+            if (dst_is_hidden_true_state)
+              os_ << 'T' << t.dst << 'T' << t.src;
+            else
+              os_  << (int)t.dst;
             if (aut_->is_univ_dest(t.dst) && highlight_edges_
-                && !opt_shared_univ_dest_)
+                && !opt_shared_univ_dest_ && !dst_is_hidden_true_state)
               {
                 auto idx = aut_->get_graph().index_of_edge(t);
                 auto iter = highlight_edges_->find(idx);
@@ -706,19 +810,20 @@ namespace spot
       void print(const const_twa_graph_ptr& aut)
       {
         aut_ = aut;
-        if (opt_want_state_names_)
+        if (opt_hide_true_states_)
+          find_true_states();
+
+        sn_ = aut->get_named_prop<std::vector<std::string>>("state-names");
+        // We have no names.  Do we have product sources?
+        if (!sn_)
           {
-            sn_ = aut->get_named_prop<std::vector<std::string>>("state-names");
-            // We have no names.  Do we have product sources?
-            if (!sn_)
-              {
-                sprod_ = aut->get_named_prop
-                  <std::vector<std::pair<unsigned, unsigned>>>
-                  ("product-states");
-                if (sprod_ && aut->num_states() != sprod_->size())
-                  sprod_ = nullptr;
-              }
+            sprod_ = aut->get_named_prop
+              <std::vector<std::pair<unsigned, unsigned>>>
+              ("product-states");
+            if (sprod_ && aut->num_states() != sprod_->size())
+              sprod_ = nullptr;
           }
+
         if (opt_orig_show_)
           orig_ = aut->get_named_prop<std::vector<unsigned>>("original-states");
         if (opt_state_labels_)
@@ -746,10 +851,22 @@ namespace spot
           aut->get_named_prop<std::map<unsigned, unsigned>>("highlight-edges");
         highlight_states_ =
           aut->get_named_prop<std::map<unsigned, unsigned>>("highlight-states");
+        if (highlight_states_)
+          {
+            // If we use more than palette_mod/2 colors, we will use
+            // some extra markers to distinguish colors.
+            auto e = highlight_states_->end();
+            highlight_states_show_num_ =
+              std::find_if(highlight_states_->begin(), e,
+                           [](auto p){
+                             return p.second >= palette_mod / 2;
+                           }) != e;
+          }
         incomplete_ =
           aut->get_named_prop<std::set<unsigned>>("incomplete-states");
+        graph_name_ = aut_->get_named_prop<std::string>("automaton-name");
         if (opt_name_)
-          name_ = aut_->get_named_prop<std::string>("automaton-name");
+          name_ = graph_name_;
         mark_states_ = (!opt_force_acc_trans_
                         && aut_->prop_state_acc().is_true());
         dcircles_ = (mark_states_
@@ -757,8 +874,10 @@ namespace spot
                      && (aut_->acc().is_buchi() || aut_->acc().is_co_buchi()));
         if (opt_shape_ == ShapeAuto)
           {
-            if (sn_ || sprod_ || aut->num_states() > 100
-                || opt_state_labels_ || orig_)
+            if ((inline_state_names_ && (sn_ || sprod_ || opt_state_labels_))
+                || (opt_state_labels_ && opt_latex_)
+                || aut->num_states() > 100
+                || orig_)
               {
                 opt_shape_ = ShapeEllipse;
                 // If all state names are short, prefer circles.
@@ -809,6 +928,8 @@ namespace spot
                   }
                 for (auto s: si->states_of(i))
                   {
+                    if (opt_hide_true_states_ && true_states_[s])
+                      continue;
                     process_state(s);
                     int trans_num = 0;
                     unsigned scc_of_s = si->scc_of(s);
@@ -829,6 +950,8 @@ namespace spot
         unsigned ns = aut_->num_states();
         for (unsigned n = 0; n < ns; ++n)
           {
+            if (opt_hide_true_states_ && true_states_[n])
+              continue;
             if (!si || !si->reachable_state(n))
               process_state(n);
             int trans_num = 0;
@@ -841,12 +964,12 @@ namespace spot
   } // anonymous namespace
 
   std::ostream&
-  print_dot(std::ostream& os, const const_twa_ptr& g,
-                  const char* options)
+  print_dot(std::ostream& os, const const_twa_ptr& g, const char* options)
   {
     dotty_output d(os, options);
-    // Enable automatic state labels for Kripke structure.
-    if (std::dynamic_pointer_cast<const fair_kripke>(g))
+    // Enable state labels for Kripke structure.
+    if (std::dynamic_pointer_cast<const fair_kripke>(g)
+        && !d.uppercase_k_seen())
       d.parse_opts("k");
     auto aut = std::dynamic_pointer_cast<const twa_graph>(g);
     if (!aut || (d.max_states_given() && aut->num_states() >= d.max_states()))
