@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2013, 2014, 2015, 2016, 2017 Laboratoire de Recherche
+// Copyright (C) 2013-2018 Laboratoire de Recherche
 // et Développement de l'Epita.
 //
 // This file is part of Spot, a model checking library.
@@ -17,6 +17,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#include "config.h"
 #include <fstream>
 #include <sstream>
 #include <map>
@@ -35,20 +36,19 @@
 #include <spot/twaalgos/sbacc.hh>
 #include <spot/twaalgos/sccfilter.hh>
 #include <spot/twaalgos/sccinfo.hh>
-#include <spot/twaalgos/stats.hh>
 
 // If you set the SPOT_TMPKEEP environment variable the temporary
 // file used to communicate with the sat solver will be left in
 // the current directory.
 //
-// Additionally, if the following DEBUG macro is set to 1, the CNF
+// Additionally, if the following TRACE macro is set to 1, the CNF
 // file will be output with a comment before each clause, and an
 // additional output file (dtwa-sat.dbg) will be created with a list
 // of all positive variables in the result and their meaning.
 
-#define DEBUG 0
+#define TRACE 0
 
-#if DEBUG
+#if TRACE
 #define dout out << "c "
 #define cnf_comment(...)  solver.comment(__VA_ARGS__)
 #define trace std::cerr
@@ -75,7 +75,7 @@ namespace spot
 
       path(unsigned src_ref)
         : src_ref(src_ref), dst_ref(src_ref),
-          acc_cand(0U), acc_ref(0U)
+          acc_cand({}), acc_ref({})
       {
       }
 
@@ -130,7 +130,7 @@ namespace spot
       auto pos = &acc.back();
       if (pos->sub.op == acc_cond::acc_op::Or)
         --pos;
-      acc_cond::mark_t all_fin = 0U;
+      acc_cond::mark_t all_fin = {};
       auto start = &acc.front();
       while (pos > start)
         {
@@ -144,8 +144,8 @@ namespace spot
             {
               // We have a conjunction of Fin and Inf sets.
               auto end = pos - pos->sub.size - 1;
-              acc_cond::mark_t fin = 0U;
-              acc_cond::mark_t inf = 0U;
+              acc_cond::mark_t fin = {};
+              acc_cond::mark_t inf = {};
               while (pos > end)
                 {
                   switch (pos->sub.op)
@@ -233,11 +233,6 @@ namespace spot
       trimming_map ref_inf_trim_map;
       trimming_map cand_inf_trim_map;
 
-      ~dict()
-      {
-        aut->get_dict()->unregister_all_my_variables(this);
-      }
-
       int
       transid(unsigned src, unsigned cond, unsigned dst)
       {
@@ -247,7 +242,7 @@ namespace spot
       int
       transid(unsigned src, bdd& cond, unsigned dst)
       {
-#if DEBUG
+#if TRACE
         try
         {
           return helper.get_t(src, alpha_map.at(cond), dst);
@@ -272,7 +267,7 @@ namespace spot
       int
       transacc(unsigned src, bdd& cond, unsigned dst, unsigned nacc)
       {
-#if DEBUG
+#if TRACE
         try
         {
           return helper.get_ta(src, alpha_map.at(cond), dst, nacc);
@@ -290,10 +285,10 @@ namespace spot
 
       int
       pathid(unsigned src_cand, unsigned src_ref, unsigned dst_cand,
-          unsigned dst_ref, acc_cond::mark_t acc_cand = 0U,
-          acc_cond::mark_t acc_ref = 0U)
+          unsigned dst_ref, acc_cond::mark_t acc_cand = {},
+          acc_cond::mark_t acc_ref = {})
       {
-#if DEBUG
+#if TRACE
         try
         {
           return helper.get_p(
@@ -315,7 +310,7 @@ namespace spot
 #endif
       }
 
-#if DEBUG
+#if TRACE
       int
       pathid(unsigned path, unsigned src_cand, unsigned dst_cand)
       {
@@ -351,8 +346,8 @@ namespace spot
 
       std::string
       fmt_p(unsigned src_cand, unsigned src_ref, unsigned dst_cand,
-          unsigned dst_ref, acc_cond::mark_t acc_cand = 0U,
-          acc_cond::mark_t acc_ref = 0U)
+          unsigned dst_ref, acc_cond::mark_t acc_cand = {},
+          acc_cond::mark_t acc_ref = {})
       {
         return helper.format_p(debug_cand_acc, debug_ref_acc, src_cand,
             src_ref, dst_cand, dst_ref, acc_cand, acc_ref);
@@ -404,7 +399,7 @@ namespace spot
         d.scc_marks.reserve(scccount);
         for (auto& v: tmp)
           {
-            acc_cond::mark_t m = 0U;
+            acc_cond::mark_t m = {};
             for (auto i: v)
               m |= i;
             d.scc_marks.emplace_back(m);
@@ -422,7 +417,7 @@ namespace spot
         d.cand_inf_trim_map = split_dnf_acc_by_inf(d.cand_acc);
 
       bdd_dict_ptr bd = aut->get_dict();
-      d.all_cand_acc.emplace_back(0U);
+      d.all_cand_acc.emplace_back(acc_cond::mark_t({}));
       for (unsigned n = 0; n < d.cand_nacc; ++n)
         {
           auto c = d.cacc.mark(n);
@@ -442,7 +437,7 @@ namespace spot
             }
         }
 
-      d.all_ref_acc.emplace_back(0U);
+      d.all_ref_acc.emplace_back(acc_cond::mark_t({}));
       unsigned ref_nacc = aut->num_sets();
       for (unsigned n = 0; n < ref_nacc; ++n)
         {
@@ -539,7 +534,7 @@ namespace spot
                           bool state_based,
                           bool colored)
     {
-#if DEBUG
+#if TRACE
       debug_dict = ref->get_dict();
 #endif
       // Compute the AP used.
@@ -572,7 +567,7 @@ namespace spot
       // Empty automaton is impossible.
       assert(d.cand_size > 0);
 
-#if DEBUG
+#if TRACE
       debug_ref_acc = &ref->acc();
       debug_cand_acc = &d.cacc;
       solver.comment("d.ref_size:", d.ref_size, '\n');
@@ -597,7 +592,7 @@ namespace spot
       for (unsigned q1 = 0; q1 < d.cand_size; ++q1)
         for (unsigned l = 0; l < alpha_size; ++l)
           {
-#if DEBUG
+#if TRACE
             solver.comment("");
             for (unsigned q2 = 0; q2 < d.cand_size; ++q2)
               {
@@ -623,7 +618,7 @@ namespace spot
         {
           unsigned nacc = d.cand_nacc;
           cnf_comment("transitions belong to exactly one of the", nacc,
-                      "acceptance set\n");
+                      "acceptance sets\n");
           for (unsigned l = 0; l < alpha_size; ++l)
             for (unsigned q1 = 0; q1 < d.cand_size; ++q1)
               for (unsigned q2 = 0; q2 < d.cand_size; ++q2)
@@ -768,7 +763,7 @@ namespace spot
 
                                           for (auto& v: missing)
                                             {
-#if DEBUG
+#if TRACE
                                               solver.comment((rejloop ?
                                                        "(11) " : "(12) "), f_p,
                                                   " ∧ ", d.fmt_t(q2, l, q3),
@@ -787,7 +782,7 @@ namespace spot
                                                   orsep = " ∨ ";
                                                 }
                                               solver.comment_rec(")\n");
-#endif // DEBUG
+#endif // TRACE
                                               solver.add({-pid, -ti});
                                               for (int s: v)
                                                 if (s < 0)
@@ -817,7 +812,7 @@ namespace spot
                                               d.cand_inf_trim
                                               (candhist_p |
                                                d.all_cand_acc[f]);
-                                            acc_cond::mark_t f2p = 0U;
+                                            acc_cond::mark_t f2p = {};
                                             if (!is_weak)
                                               f2p = d.ref_inf_trim(refhist |
                                                                    curacc);
@@ -825,7 +820,7 @@ namespace spot
                                                 q1, q1p, q3, dp, f2, f2p);
                                             if (pid == p2id)
                                               continue;
-#if DEBUG
+#if TRACE
                                             solver.comment("(13) ", f_p, " ∧ ",
                                                  d.fmt_t(q1, l, q3), "δ ");
 
@@ -883,7 +878,7 @@ namespace spot
       a->set_acceptance(satdict.cand_nacc, satdict.cand_acc);
       a->new_states(satdict.cand_size);
 
-#if DEBUG
+#if TRACE
       std::fstream out("dtwa-sat.dbg",
                        std::ios_base::trunc | std::ios_base::out);
       out.exceptions(std::ifstream::failbit | std::ifstream::badbit);
@@ -905,7 +900,7 @@ namespace spot
               // Skip (s,l,d2) if we have already seen some (s,l,d1).
               if (seen_trans.insert(src_cond(i, satdict.alpha_vect[j])).second)
               {
-                acc_cond::mark_t acc = 0U;
+                acc_cond::mark_t acc = {};
                 if (state_based)
                   {
                     auto tmp = state_acc.find(i);
@@ -922,7 +917,7 @@ namespace spot
             }
           }
 
-#if DEBUG
+#if TRACE
       dout << "--- transition variables ---\n";
       for (unsigned i = 0; i < satdict.cand_size; ++i)
         for (unsigned j = 0; j < alpha_size; ++j)
@@ -981,6 +976,7 @@ namespace spot
 #endif
 
       a->merge_edges();
+      a->purge_unreachable_states();
       return a;
     }
   }
@@ -1022,7 +1018,7 @@ namespace spot
     if (!solution.second.empty())
       res = sat_build(solution.second, d, a, state_based);
 
-    print_log(t, target_state_number, res, solver); // if SPOT_SATLOG is set.
+    print_log(t, a->num_states(), target_state_number, res, solver);
 
     trace << "dtwa_sat_synthetize(...) = " << res << '\n';
     return res;
@@ -1065,7 +1061,6 @@ namespace spot
   dichotomy_dtwa_research(int max,
                           dict& d,
                           satsolver& solver,
-                          timer_map& t1,
                           const_twa_graph_ptr& prev,
                           bool state_based)
   {
@@ -1080,35 +1075,46 @@ namespace spot
       trace << "min:" << min << ", max:" << max << ", target:" << target
         << '\n';
 
+      timer_map t1;
+      t1.start("encode");
       solver.assume(d.nvars + target);
+      t1.stop("encode");
       trace << "solver.assume(" << d.nvars + target << ")\n";
-
+      t1.start("solve");
       satsolver::solution_pair solution = solver.get_solution();
+      t1.stop("solve");
       if (solution.second.empty())
       {
         trace << "UNSAT\n";
         max = target;
+        print_log(t1, prev->num_states(), d.cand_size - target,
+                  nullptr, solver);
       }
       else
       {
         trace << "SAT\n";
         res = sat_build(solution.second, d, prev, state_based);
-        min = d.cand_size - stats_reachable(res).states + 1;
+        min = d.cand_size - res->num_states() + 1;
+        print_log(t1, prev->num_states(), d.cand_size - target,
+                  res, solver);
       }
     }
 
     trace << "End with max:" << max << ", min:" << min << '\n';
     if (!res)
     {
-      trace << "All assumptions are UNSAT, let's try without...";
+      trace << "All assumptions are UNSAT, let's try without...\n";
+      timer_map t1;
+      t1.start("encode");
+      t1.stop("encode");
+      t1.start("solve");
       satsolver::solution_pair solution = solver.get_solution();
+      t1.stop("solve");
       trace << (solution.second.empty() ? "UNSAT!\n" : "SAT\n");
       res = solution.second.empty() ? nullptr :
         sat_build(solution.second, d, prev, state_based);
+      print_log(t1, prev->num_states(), d.cand_size - target, res, solver);
     }
-
-    t1.stop("solve");
-    print_log(t1, d.cand_size - target , res, solver); // SPOT_SATLOG.
     return res ? res : std::const_pointer_cast<spot::twa_graph>(prev);
   }
 
@@ -1127,8 +1133,7 @@ namespace spot
 
     const_twa_graph_ptr prev = a;
     dict d(prev);
-    d.cand_size = (max_states < 0) ?
-      stats_reachable(prev).states - 1 : max_states;
+    d.cand_size = (max_states < 0) ? prev->num_states() - 1 : max_states;
     d.cand_nacc = target_acc_number;
     d.cand_acc = target_acc;
     if (d.cand_size == 0)
@@ -1143,9 +1148,9 @@ namespace spot
     while (next && d.cand_size > 0)
     {
       // Warns the satsolver of the number of assumptions.
-      int n_assumptions = (int) d.cand_size < sat_incr_steps ?
+      int n_assumptions = (int) d.cand_size <= sat_incr_steps ?
         d.cand_size - 1 : sat_incr_steps;
-      trace << "number of assumptions:" << n_assumptions << '\n';
+      trace << "number of assumptions: " << n_assumptions << '\n';
       satsolver solver;
       solver.set_nassumptions_vars(n_assumptions);
 
@@ -1181,35 +1186,39 @@ namespace spot
           solver.add({-assume_lit, assume_lit - 1, 0});
         }
       }
-      t1.stop("encode");
 
-      t1.start("solve");
       if (n_assumptions)
       {
         trace << "solver.assume(" << d.nvars + n_assumptions << ")\n";
         solver.assume(d.nvars + n_assumptions);
       }
+      t1.stop("encode");
+      t1.start("solve");
       satsolver::solution_pair solution = solver.get_solution();
+      t1.stop("solve");
 
       if (solution.second.empty() && n_assumptions) // UNSAT
       {
+        print_log(t1, prev->num_states(),
+                  d.cand_size - n_assumptions, nullptr, solver);
         trace << "UNSAT\n";
         twa_graph_ptr res = dichotomy_dtwa_research(n_assumptions, d, solver,
-                                                    t1, prev, state_based);
+                                                    prev, state_based);
         return res == a ? nullptr : res;
       }
 
-      t1.stop("solve");
       trace << "SAT, restarting from zero\n";
       next = solution.second.empty() ? nullptr :
         sat_build(solution.second, d, prev, state_based);
-      print_log(t1, d.cand_size - n_assumptions, next, solver); // SPOT_SATLOG.
+      print_log(t1, prev->num_states(),
+                d.cand_size - n_assumptions, next, solver);
 
       if (next)
       {
         prev = next;
+        trace << "prev has " << prev->num_states() << '\n';
         d = dict(prev);
-        d.cand_size = stats_reachable(prev).states - 1;
+        d.cand_size = prev->num_states() - 1;
         d.cand_nacc = target_acc_number;
         d.cand_acc = target_acc;
         if (d.cand_size == 0)
@@ -1232,7 +1241,7 @@ namespace spot
 
     dict d(prev);
     d.cand_size = (max_states < 0) ?
-      stats_reachable(prev).states - 1 : max_states;
+      prev->num_states() - 1 : max_states;
     d.cand_nacc = target_acc_number;
     d.cand_acc = target_acc;
     if (d.cand_size == 0)
@@ -1259,9 +1268,9 @@ namespace spot
       t1.stop("solve");
       next = solution.second.empty() ? nullptr :
         sat_build(solution.second, d, prev, state_based);
-      print_log(t1, d.cand_size, next, solver); // If SPOT_SATLOG is set.
+      print_log(t1, prev->num_states(), d.cand_size, next, solver);
 
-      trace << "First iteraton done\n";
+      trace << "First iteration done\n";
 
       // Compute the AP used.
       bdd ap = prev->ap_vars();
@@ -1272,27 +1281,25 @@ namespace spot
       for (int k = 0; next && d.cand_size > 0 && (naive || k < sat_incr_steps);
            ++k)
       {
-        t1.start("encode");
         prev = next;
-        int reach_states = stats_reachable(prev).states;
-        cnf_comment("Next iteration:", reach_states - 1, "\n");
-
+        int reach_states = prev->num_states();
         trace << "Encoding the deletion of state " << reach_states - 1 << '\n';
-
+        cnf_comment("Next iteration:", reach_states - 1, "\n");
+        timer_map t2;
+        t2.start("encode");
         // Add new constraints.
         for (unsigned i = reach_states - 1; i < d.cand_size; ++i)
           for (unsigned l = 0; l < alpha_size; ++l)
             for (unsigned j = 0; j < orig_cand_size; ++j)
               solver.add({-d.transid(j, l, i), 0});
-
+        t2.stop("encode");
         d.cand_size = reach_states - 1;
-        t1.stop("encode");
-        t1.start("solve");
+        t2.start("solve");
         solution = solver.get_solution();
-        t1.stop("solve");
+        t2.stop("solve");
         next = solution.second.empty() ? nullptr :
           sat_build(solution.second, d, prev, state_based);
-        print_log(t1, d.cand_size, next, solver); // If SPOT_SATLOG is set.
+        print_log(t2, prev->num_states(), d.cand_size, next, solver);
       }
 
       if (next)
@@ -1300,7 +1307,7 @@ namespace spot
         trace << "Starting from scratch\n";
         prev = next;
         d = dict(prev);
-        d.cand_size = stats_reachable(prev).states - 1;
+        d.cand_size = prev->num_states() - 1;
         d.cand_nacc = target_acc_number;
         d.cand_acc = target_acc;
         if (d.cand_size == 0)
@@ -1319,8 +1326,7 @@ namespace spot
                     bool state_based, int max_states,
                     bool colored)
   {
-    int n_states = (max_states < 0) ?
-      stats_reachable(a).states : max_states + 1;
+    int n_states = (max_states < 0) ? a->num_states() : max_states + 1;
 
     twa_graph_ptr prev = nullptr;
     for (;;)
@@ -1332,7 +1338,7 @@ namespace spot
         if (!next)
           return prev;
         else
-          n_states = stats_reachable(next).states;
+          n_states = next->num_states();
         prev = next;
       }
     SPOT_UNREACHABLE();
@@ -1347,7 +1353,7 @@ namespace spot
   {
     trace << "Dichotomy\n";
     if (max_states < 1)
-      max_states = stats_reachable(a).states - 1;
+      max_states = a->num_states() - 1;
     int min_states = 1;
     if (langmap)
     {
@@ -1372,7 +1378,7 @@ namespace spot
         else
           {
             prev = next;
-            max_states = stats_reachable(next).states - 1;
+            max_states = next->num_states() - 1;
           }
       }
     return prev;
@@ -1400,10 +1406,10 @@ namespace spot
     bool sat_langmap = om.get("sat-langmap", 0);
     int states = om.get("states", -1);
     int max_states = om.get("max-states", -1);
-    auto accstr = om.get_str("acc");
+    std::string accstr = om.get_str("acc");
     bool colored = om.get("colored", 0);
     int preproc = om.get("preproc", 3);
-
+    set_satlog_filename(om.get_str("log"));
 
     // Set default sat-incr-steps value if not provided and used.
     if (sat_incr == 1 && !sat_incr_steps) // Assume
@@ -1550,7 +1556,7 @@ namespace spot
 
     if (a)
       {
-        if (state_based)
+        if (state_based || colored)
           a = scc_filter_states(a);
         else
           a = scc_filter(a);
