@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2008, 2011, 2014, 2015, 2016 Laboratoire de Recherche
+// Copyright (C) 2008, 2011, 2014-2016, 2018 Laboratoire de Recherche
 // et Développement de l'Epita (LRDE).
 // Copyright (C) 2003, 2004, 2005, 2006 Laboratoire d'Informatique de
 // Paris 6 (LIP6), département Systèmes Répartis Coopératifs (SRC),
@@ -20,11 +20,13 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-// #define TRACE
 
+#include "config.h"
 #include <iostream>
+// #define TRACE
 #ifdef TRACE
 #define trace std::cerr
+#define SPOT_TRACE 1
 #else
 #define trace while (0) std::cerr
 #endif
@@ -161,7 +163,7 @@ namespace spot
       const state* init = ecs_->aut->get_init_state();
       ecs_->h[init] = 1;
       ecs_->root.push(1);
-      arc.push(0U);
+      arc.push({});
       twa_succ_iterator* iter = ecs_->aut->succ_iter(init);
       iter->first();
       todo.emplace(init, iter);
@@ -369,7 +371,7 @@ namespace spot
     assert(depth() == 0);
   }
 
-  void
+#ifdef TRACE
   couvreur99_check_shy::dump_queue(std::ostream& os)
   {
     os << "--- TODO ---\n";
@@ -389,6 +391,7 @@ namespace spot
         os << "}\n";
       }
   }
+#endif
 
   emptiness_check_result_ptr
   couvreur99_check_shy::check()
