@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2008, 2010, 2011, 2013-2017 Laboratoire de
+// Copyright (C) 2008, 2010, 2011, 2013-2018 Laboratoire de
 // recherche et développement de l'Epita (LRDE).
 // Copyright (C) 2004, 2005 Laboratoire d'Informatique de Paris 6
 // (LIP6), département Systèmes Répartis Coopératifs (SRC), Université
@@ -22,6 +22,7 @@
 
 //#define TRACE
 
+#include "config.h"
 #include <iostream>
 #ifdef TRACE
 #define trace std::cerr
@@ -363,7 +364,7 @@ namespace spot
             virtual bool
             match(twa_run::step& step, const state*) override
             {
-              return step.acc != 0U;
+              return !!step.acc;
             }
           };
 
