@@ -1,9 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2011, 2012, 2014, 2015, 2016, 2017 Laboratoire de Recherche et
+// Copyright (C) 2014-2018 Laboratoire de Recherche et
 // Developpement de l'Epita (LRDE).
-// Copyright (C) 2003, 2004  Laboratoire d'Informatique de Paris 6 (LIP6),
-// département Systèmes Répartis Coopératifs (SRC), Université Pierre
-// et Marie Curie.
 //
 // This file is part of Spot, a model checking library.
 //
@@ -20,6 +17,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#include "config.h"
 #include <ostream>
 #include <sstream>
 #include <cstring>
@@ -27,7 +25,6 @@
 #include <spot/twa/twa.hh>
 #include <spot/twa/twagraph.hh>
 #include <spot/twaalgos/hoa.hh>
-#include <spot/twaalgos/reachiter.hh>
 #include <spot/misc/escape.hh>
 #include <spot/misc/bddlt.hh>
 #include <spot/misc/minato.hh>
@@ -74,7 +71,7 @@ namespace spot
       emit_acc(std::ostream& os, acc_cond::mark_t b)
       {
         // FIXME: We could use a cache for this.
-        if (b == 0U)
+        if (!b)
           return os;
         os << " {";
         bool notfirst = false;
@@ -106,7 +103,7 @@ namespace spot
             bdd available = bddtrue;
             bool st_acc = true;
             bool notfirst = false;
-            acc_cond::mark_t prev = 0U;
+            acc_cond::mark_t prev = {};
             bool has_succ = false;
             bdd lastcond = bddfalse;
             for (auto& t: aut->out(src))
@@ -638,7 +635,7 @@ namespace spot
           os << " \"" << (*sn)[i] << '"';
         if (this_acc == Hoa_Acceptance_States)
           {
-            acc_cond::mark_t acc = 0U;
+            acc_cond::mark_t acc = {};
             for (auto& t: aut->out(i))
               {
                 acc = t.acc;
