@@ -17,6 +17,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#include "config.h"
 #include <spot/twaalgos/postproc.hh>
 #include <spot/twaalgos/minimize.hh>
 #include <spot/twaalgos/simulation.hh>
@@ -37,8 +38,8 @@
 #include <spot/twaalgos/alternation.hh>
 #include <spot/twaalgos/parity.hh>
 #include <spot/twaalgos/cobuchi.hh>
-#include <spot/twaalgos/dot.hh>
 #include <spot/twaalgos/rabin2parity.hh>
+#include <spot/twaalgos/cleanacc.hh>
 
 namespace spot
 {
@@ -104,7 +105,7 @@ namespace spot
   }
 
   twa_graph_ptr
-  postprocessor::do_simul(const twa_graph_ptr& a, int opt)
+  postprocessor::do_simul(const twa_graph_ptr& a, int opt) const
   {
     if (!has_separate_sets(a))
       return a;
@@ -123,7 +124,7 @@ namespace spot
   }
 
   twa_graph_ptr
-  postprocessor::do_sba_simul(const twa_graph_ptr& a, int opt)
+  postprocessor::do_sba_simul(const twa_graph_ptr& a, int opt) const
   {
     if (ba_simul_ <= 0)
       return a;
@@ -142,7 +143,7 @@ namespace spot
   }
 
   twa_graph_ptr
-  postprocessor::do_degen(const twa_graph_ptr& a)
+  postprocessor::do_degen(const twa_graph_ptr& a) const
   {
     auto d = degeneralize(a,
                           degen_reset_, degen_order_,
@@ -152,7 +153,7 @@ namespace spot
   }
 
   twa_graph_ptr
-  postprocessor::do_degen_tba(const twa_graph_ptr& a)
+  postprocessor::do_degen_tba(const twa_graph_ptr& a) const
   {
     return degeneralize_tba(a,
                             degen_reset_, degen_order_,
@@ -161,21 +162,18 @@ namespace spot
   }
 
   twa_graph_ptr
-  postprocessor::do_scc_filter(const twa_graph_ptr& a, bool arg)
+  postprocessor::do_scc_filter(const twa_graph_ptr& a, bool arg) const
   {
     if (scc_filter_ == 0)
       return a;
-    // If the automaton is weak, using transition-based acceptance
-    // won't help, so let's preserve state-based acceptance.
-    if ((state_based_ || a->prop_inherently_weak().is_true())
-        && a->prop_state_acc().is_true())
+    if (state_based_ && a->prop_state_acc().is_true())
       return scc_filter_states(a, arg);
     else
       return scc_filter(a, arg);
   }
 
   twa_graph_ptr
-  postprocessor::do_scc_filter(const twa_graph_ptr& a)
+  postprocessor::do_scc_filter(const twa_graph_ptr& a) const
   {
     return do_scc_filter(a, scc_filter_ > 1);
   }
@@ -185,6 +183,36 @@ namespace spot
 #define SBACC_ (pref_ & SBAcc)
 #define COLORED_ (pref_ & Colored)
 
+
+  twa_graph_ptr
+  postprocessor::finalize(twa_graph_ptr tmp) const
+  {
+    if (COMP_)
+      tmp = complete(tmp);
+    bool want_parity = type_ & Parity;
+    if (want_parity && tmp->acc().is_generalized_buchi())
+      tmp = SBACC_ ? do_degen(tmp) : do_degen_tba(tmp);
+    if (SBACC_)
+      tmp = sbacc(tmp);
+    if (want_parity)
+      {
+        if (COLORED_)
+          colorize_parity_here(tmp);
+        parity_kind kind = parity_kind_any;
+        parity_style style = parity_style_any;
+        if ((type_ & ParityMin) == ParityMin)
+          kind = parity_kind_min;
+        else if ((type_ & ParityMax) == ParityMax)
+          kind = parity_kind_max;
+        if ((type_ & ParityOdd) == ParityOdd)
+          style = parity_style_odd;
+        else if ((type_ & ParityEven) == ParityEven)
+          style = parity_style_even;
+        change_parity_here(tmp, kind, style);
+      }
+    return tmp;
+  }
+
   twa_graph_ptr
   postprocessor::run(twa_graph_ptr a, formula f)
   {
@@ -198,37 +226,31 @@ namespace spot
       state_based_ = true;
 
     bool via_gba = (type_ == BA) || (type_ == TGBA) || (type_ == Monitor);
-    bool want_parity = (type_ & Parity) == Parity;
+    bool want_parity = type_ & Parity;
     if (COLORED_ && !want_parity)
       throw std::runtime_error("postprocessor: the Colored setting only works "
                                "for parity acceptance");
 
-    auto finalize = [&](twa_graph_ptr tmp)
+
+
+    // Attempt to simplify the acceptance condition, unless this is a
+    // parity automaton and we want parity acceptance in the output.
+    auto simplify_acc = [&](twa_graph_ptr in)
       {
-        if (COMP_)
-          tmp = complete(tmp);
-        if (want_parity && tmp->acc().is_generalized_buchi())
-          tmp = SBACC_ ? do_degen(tmp) : do_degen_tba(tmp);
-        if (SBACC_)
-          tmp = sbacc(tmp);
-        if (want_parity)
+        if (PREF_ == Any && level_ == Low)
+          return in;
+        if (!(want_parity && in->acc().is_parity()))
           {
-            if (COLORED_)
-              colorize_parity_here(tmp);
-            parity_kind kind = parity_kind_any;
-            parity_style style = parity_style_any;
-            if ((type_ & ParityMin) == ParityMin)
-              kind = parity_kind_min;
-            else if ((type_ & ParityMax) == ParityMax)
-              kind = parity_kind_max;
-            if ((type_ & ParityOdd) == ParityOdd)
-              style = parity_style_odd;
-            else if ((type_ & ParityEven) == ParityEven)
-              style = parity_style_even;
-            change_parity_here(tmp, kind, style);
+            if (level_ == High)
+              return simplify_acceptance(in);
+            else
+              return cleanup_acceptance(in);
           }
-        return tmp;
+        if (level_ == High)
+          return cleanup_parity(in);
+        return in;
       };
+    a = simplify_acc(a);
 
     if (!a->is_existential() &&
         // We will probably have to revisit this condition later.
@@ -438,6 +460,7 @@ namespace spot
       {
         dba = tgba_determinize(to_generalized_buchi(sim),
                                false, det_scc_, det_simul_, det_stutter_);
+        dba = simplify_acc(dba);
         if (level_ != Low)
           dba = simulation(dba);
         sim = nullptr;
