@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2014-2017 Laboratoire de Recherche et Développement
+// Copyright (C) 2014-2018 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -17,6 +17,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#include "config.h"
 #include <spot/twaalgos/sccinfo.hh>
 #include <stack>
 #include <algorithm>
@@ -57,8 +58,9 @@ namespace spot
       }
 
       acc_cond::mark_t in_acc; // Acceptance sets on the incoming transition
-      acc_cond::mark_t acc = 0U; // union of all acceptance marks in the SCC
-      acc_cond::mark_t common = -1U; // intersection of all marks in the SCC
+      acc_cond::mark_t acc = {}; // union of all acceptance marks in the SCC
+      // intersection of all marks in the SCC
+      acc_cond::mark_t common = acc_cond::mark_t::all();
       int index;                     // Index of the SCC
       bool trivial = true;           // Whether the SCC has no cycle
       bool accepting = false;        // Necessarily accepting
@@ -204,7 +206,7 @@ namespace spot
           continue;
         assert(spi == 0);
         h_[init] = --num_;
-        root_.emplace_back(num_, 0U);
+        root_.emplace_back(num_, acc_cond::mark_t({}));
         todo_.emplace(stack_item{init, gr.state_storage(init).succ, 0});
         live.emplace_back(init);
 
@@ -530,6 +532,7 @@ namespace spot
         cur.assign(aut_->num_states(), false);
 
         auto copy = make_twa_graph(aut_->get_dict());
+        copy->copy_ap_of(aut_);
         copy->copy_acceptance_of(aut_);
         copy->prop_state_acc(aut_->prop_state_acc());
         transform_accessible(aut_, copy, [&](unsigned src,
@@ -574,7 +577,7 @@ namespace spot
 
         // Get all Fin acceptance set that appears in the SCC and does not have
         // their corresponding Inf appearing in the SCC.
-        acc_cond::mark_t m = 0u;
+        acc_cond::mark_t m = {};
         if (fin)
           for (unsigned p = 0; p < nb_pairs; ++p)
             if (fin & pairs[p].fin && !(inf & pairs[p].inf))
@@ -623,8 +626,8 @@ namespace spot
         for (unsigned i = 0; i < nb_states; ++i)
           old.push_back(i);
 
-        acc_cond::mark_t all_fin = 0U;
-        acc_cond::mark_t all_inf = 0U;
+        acc_cond::mark_t all_fin = {};
+        acc_cond::mark_t all_inf = {};
         std::tie(all_inf, all_fin) = aut_->get_acceptance().used_inf_fin_sets();
 
         states_on_acc_cycle_of_rec(scc, all_fin, all_inf, nb_pairs, pairs, res,
