@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2011, 2013-2017  Laboratoire de Recherche et
+// Copyright (C) 2011, 2013-2018  Laboratoire de Recherche et
 // Développement de l'Epita (LRDE).
 // Copyright (C) 2004, 2005  Laboratoire d'Informatique de Paris 6 (LIP6),
 // département Systèmes Répartis Coopératifs (SRC), Université Pierre
@@ -22,6 +22,7 @@
 
 //#define TRACE
 
+#include "config.h"
 #include <iostream>
 #ifdef TRACE
 #define trace std::cerr
@@ -100,7 +101,7 @@ namespace spot
             const state* s0 = a_->get_init_state();
             inc_states();
             h.add_new_state(s0, CYAN);
-            push(st_blue, s0, bddfalse, 0U);
+            push(st_blue, s0, bddfalse, {});
             if (dfs_blue())
               return std::make_shared<se05_result>(t, options());
           }
