@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012-2017 Laboratoire de Recherche et Développement
+// Copyright (C) 2012-2018 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -17,6 +17,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#include "config.h"
 #include <queue>
 #include <map>
 #include <utility>
@@ -25,7 +26,6 @@
 #include <spot/twaalgos/simulation.hh>
 #include <spot/misc/minato.hh>
 #include <spot/twa/bddprint.hh>
-#include <spot/twaalgos/reachiter.hh>
 #include <spot/twaalgos/sccfilter.hh>
 #include <spot/twaalgos/sepsets.hh>
 #include <spot/twaalgos/isdet.hh>
@@ -192,7 +192,7 @@ namespace spot
                         // the destination state on its incoming arcs
                         // (which now become outgoing arcs after
                         // transposition).
-                        acc = 0U;
+                        acc = {};
                         for (auto& td: in->out(t.dst))
                           {
                             acc = td.acc ^ all_inf;
@@ -475,7 +475,7 @@ namespace spot
         // Acceptance of states.  Only used if Sba && Cosimulation.
         std::vector<acc_cond::mark_t> accst;
         if (Sba && Cosimulation)
-          accst.resize(res->num_states(), 0U);
+          accst.resize(res->num_states(), acc_cond::mark_t({}));
 
         stat.states = bdd_lstate_.size();
         stat.edges = 0;
@@ -564,7 +564,7 @@ namespace spot
                             // can't do this here, store this in a table
                             // so we can fix it later.
                             accst[gb->get_state(src.id())] = acc;
-                            acc = 0U;
+                            acc = {};
                           }
                         gb->new_edge(dst.id(), src.id(), cond, acc);
                       }
@@ -591,7 +591,7 @@ namespace spot
             for (unsigned s = 0; s < ns; ++s)
               {
                 acc_cond::mark_t acc = accst[s];
-                if (acc == 0U)
+                if (!acc)
                   continue;
                 for (auto& t: res->out(s))
                   t.acc = acc;
