@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2011, 2013-2017  Laboratoire de Recherche et
+// Copyright (C) 2011, 2013-2018  Laboratoire de Recherche et
 // Developpement de l'Epita (LRDE).
 // Copyright (C) 2004, 2005  Laboratoire d'Informatique de Paris 6 (LIP6),
 // département Systèmes Répartis Coopératifs (SRC), Université Pierre
@@ -25,6 +25,7 @@
 
 //#define TRACE
 
+#include "config.h"
 #include <iostream>
 #ifdef TRACE
 #define trace std::cerr
@@ -92,7 +93,7 @@ namespace spot
         const state* s0 = a_->get_init_state();
         inc_states();
         h.add_new_state(s0, BLUE);
-        push(st_blue, s0, bddfalse, 0U);
+        push(st_blue, s0, bddfalse, {});
         auto t = std::static_pointer_cast<tau03_search>
           (this->emptiness_check::shared_from_this());
         if (dfs_blue())
@@ -298,7 +299,7 @@ namespace spot
           }
         acc_cond::mark_t get_acc() const
           {
-            assert(!is_white());
+            SPOT_ASSUME(!is_white());
             return *acc;
           }
         void cumulate_acc(acc_cond::mark_t a)
@@ -349,7 +350,7 @@ namespace spot
           assert(h.find(s) == h.end());
           h.emplace(std::piecewise_construct,
                     std::forward_as_tuple(s),
-                    std::forward_as_tuple(c, 0U));
+                    std::forward_as_tuple(c, acc_cond::mark_t({})));
         }
 
       void pop_notify(const state*) const
