@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-x
-// Copyright (C) 2014, 2015, 2017 Laboratoire de Recherche et
+// Copyright (C) 2014, 2015, 2017, 2018 Laboratoire de Recherche et
 // Développement de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -17,11 +17,13 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#include "config.h"
 #include <iostream>
 #include <iterator>
 #include <vector>
 #include <cassert>
 #include <cstdlib>
+#include <cstring>
 #include <spot/twa/acc.hh>
 
 static void check(spot::acc_cond& ac, spot::acc_cond::mark_t m)
@@ -62,8 +64,13 @@ int main()
   auto m1 = spot::acc_cond::mark_t({0, 2});
   auto m2 = spot::acc_cond::mark_t({0, 3});
   auto m3 = spot::acc_cond::mark_t({2, 1});
+  auto m4 =
+    spot::acc_cond::mark_t({0, spot::acc_cond::mark_t::max_accsets() - 2});
+  if (!((m4.min_set() == 1) &&
+        (m4.max_set() == spot::acc_cond::mark_t::max_accsets() - 1)))
+    return 1;
 
-  spot::acc_cond::mark_t m0 = 0U;
+  spot::acc_cond::mark_t m0 = {};
   std::cout << m0.max_set() << ' ' << m0.min_set() << '\n';
   std::cout << m3.max_set() << ' ' << m3.min_set() << '\n';
 
@@ -159,7 +166,7 @@ int main()
   // {1}
   // {2, 3}
   std::cout << code3 << ' ' << "{0} true\n";
-  spot::acc_cond::mark_t m = 0U;
+  spot::acc_cond::mark_t m = {};
   m.set(0);
   print(code3.missing(m, true));
   std::cout << code3 << ' ' << "{0} false\n";
@@ -177,4 +184,49 @@ int main()
   std::cout << c1 << '\n';
   std::cout << c2 << '\n';
   assert(c1 == c2);
+
+  try
+    {
+      spot::acc_cond a{spot::acc_cond::mark_t::max_accsets() + 1};
+    }
+  catch (const std::runtime_error& e)
+    {
+      assert(!std::strncmp(e.what(), "Too many acceptance sets used.", 30));
+    }
+
+#if SPOT_DEBUG
+  // Those error message are disabled in non-debugging code as
+  // shifting mark_t is usually done in the innermost loop of
+  // algorithms.  However, they are still done in Python, and we
+  // test them in python/except.py
+  try
+    {
+      spot::acc_cond::mark_t m{0};
+      m <<= spot::acc_cond::mark_t::max_accsets() + 1;
+    }
+  catch (const std::runtime_error& e)
+    {
+      assert(!std::strncmp(e.what(), "Too many acceptance sets used.", 30));
+    }
+
+  try
+    {
+      spot::acc_cond::mark_t m{0};
+      m >>= spot::acc_cond::mark_t::max_accsets() + 1;
+    }
+  catch (const std::runtime_error& e)
+    {
+      assert(!std::strncmp(e.what(), "Too many acceptance sets used.", 30));
+    }
+  try
+    {
+      spot::acc_cond::mark_t m{spot::acc_cond::mark_t::max_accsets()};
+    }
+  catch (const std::runtime_error& e)
+    {
+      assert(!std::strcmp(e.what(), "bit index is out of bounds"));
+    }
+#endif
+
+  return 0;
 }
