@@ -28,9 +28,9 @@ AP: 1 "a"
 acc-name: co-Buchi
 Acceptance: 1 Fin(0)
 properties: trans-labels explicit-labels state-acc deterministic
-properties: inherently-weak
+properties: very-weak
 --BODY--
-State: 0
+State: 0 {0}
 [0] 1
 State: 1
 [t] 1
